// fmcAP.cxx	-- commands the autopilot system
//				-- manage vnav phases
//
// Written by Harald Johnsen, started Aug 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include "fmc.hxx"
#include "fmcdata.hxx"
#include <simgear/debug/logstream.hxx>
#include <simgear/math/sg_geodesy.hxx>
#include <simgear/constants.h>
#include "fmcpages.hxx"

#include <math.h>

bool vnav_was_rearmed=true;
bool lnav_was_rearmed=true;
bool vnav_active=false;
bool lnav_active=false;
bool no_route = true;
static float previous_distto_wp=0;
bool fg_vnav_active;
bool fg_lnav_active;

void vnav_update_phase(void)
{
	string operatingPhase = DataGet("fmc/internal/phase");
	string newPhase = operatingPhase;
	float crzalt = stof( DataGet("fmc/perf/crz-alt") );
	float acalt = stof( DataGet("fmc/fmc-pos-alt") );
	if( crzalt == 0.0)
		return;

	if( operatingPhase == "CLB" && fabs( acalt - crzalt ) < 500.0 )
	{
		newPhase = "CRZ";
	    SG_LOG(SG_GENERAL, SG_INFO, "VNAV:from CLB to CRZ");
	}
	else if( operatingPhase == "TO" && acalt >= 400.0 )
	{
		newPhase = "TO2";
	    SG_LOG(SG_GENERAL, SG_INFO, "VNAV:from TO to TO2");
	}
	if( (operatingPhase == "TO" || operatingPhase == "TO2") && acalt >= 1200.0 )
	{
		newPhase = "CLB";
	    SG_LOG(SG_GENERAL, SG_INFO, "VNAV:from TOx to CLB");
	}

	DataSet("fmc/internal/phase", newPhase);

	if( newPhase == operatingPhase && !vnav_was_rearmed )
		return;
	vnav_was_rearmed = false;

	if( newPhase == "TO2" )
	{
		DataSet("fmc/autopilot/active-spd-mode", "FMC");
		DataSet("fmc/autopilot/active-spd", "200");	//V2+15
	}
	if( newPhase == "CLB" )
	{
		DataSet("fmc/autopilot/active-spd-mode", "FMC");
		DataSet("fmc/autopilot/active-spd", "250");	// 250 | Vref+80
	}
	if( newPhase == "CRZ" )
	{
		DataSet("fmc/autopilot/active-spd-mode", "ECON");
		DataSet("fmc/autopilot/active-spd", DataGet("fmc/perf/econ-spd") );
	}
}

void lnav_update(void)
{
	string dest = DataGet( "fmc/legact/wp[1]/id", false);
	string pos = DataGet("fmc/legact/wp[1]/pos", false);
	string alt = DataGet("fmc/legact/wp[1]/alt", false);
	string spd = DataGet("fmc/legact/wp[1]/spd", false);
	if(dest.empty())
	{
		no_route = true;
		previous_distto_wp = 9e9;
		return;
	}
	no_route = false;
	float lat, lon, reflat, reflon;
	extract_latlon(pos, &lat, &lon);
	pos = DataGet("fmc/fmc-pos", false);
	extract_latlon ( pos, &reflat, &reflon );

	double az1, az2, d;
	float ac_alt = stof(DataGet("fg/position/altitude-ft", false));
	geo_inverse_wgs_84(ac_alt, reflat, reflon, lat, lon, &az1, &az2, &d);
	d *= SG_METER_TO_NM;

	if( ((d-0.01) > previous_distto_wp && d < 20.0) || (d < 1.0) )
	{
		// the distance increase, this means we have passed this wp
		previous_distto_wp = 9e9;

		// pop this wp
		fmcPage->shiftWP();
	}
	DataSet("fmc/autopilot/active-hdg", ftos( az1 ) );

	previous_distto_wp = d;
}

//		"fg/autopilot/gui/hdg-lnav", "fg/autopilot/gui/alt-vnav", "fg/autopilot/gui/vel-vnav",
//		"fg/autopilot/gui/hdg-active", "fg/autopilot/gui/alt-active", "fg/autopilot/gui/vel-active",

void AP_request_update(void)
{
	lnav_was_rearmed=true;
	vnav_was_rearmed=true;
}

void AP_update(void)
{
	fg_vnav_active = DataGet("fg/autopilot/gui/alt-vnav") == "true";
	fg_lnav_active = DataGet("fg/autopilot/gui/hdg-lnav") == "true";

	if(fg_lnav_active && ! lnav_active)
		lnav_was_rearmed=true;
	if(fg_vnav_active && ! vnav_active)
		vnav_was_rearmed=true;

	vnav_update_phase();
	lnav_update();
	if( fg_lnav_active && DataGet("fg/autopilot/gui/hdg-active") == "true" && ! no_route)
	{
		DataSet( "/autopilot/settings/true-heading-deg", DataGet("fmc/autopilot/active-hdg", false) );
		DataSet( "/autopilot/locks/heading", "true-heading-hold" );
	}
	if( fg_vnav_active && DataGet("fg/autopilot/gui/alt-active") == "true" )
	{
	}
}
