// fmcData.cxx -- a library that manage a list of properties
//
// Written by Harald Johnsen, started Juilly 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include "fmc.hxx"
#include <simgear/misc/strutils.hxx>
#include <map>

SG_USING_STD(map);

#include "fmcdata.hxx"
#include "fmcnav.hxx"
#include "fmcpages.hxx"
#include "fmccomm.hxx"

typedef struct 
{
    string value;
    dataType type;
} dataEntry;

typedef map < string, dataEntry > data_map_type;
static bool fixUnique;
data_map_type dataPool;

static dataEntry *FindData(string name)
{
	return &dataPool[name];
}

//===========================================================================
//DataFixUnique
//			value	= if true then a nav aid must be unique
//
void DataFixUnique(bool value)
{
    fixUnique = value;
}

string latlon(float lat, float lon)
{
	char temp[30];
	sprintf(temp, "%f/%f", lat, lon);
	return temp;
}

void extract_latlon(string data, float *lat, float *lon)
{
	sscanf(data.c_str(), "%f/%f", lat, lon);
}

//===========================================================================
//DataSet
//			name	= name of the property
//			value
//			theType	= optional, used to check type consistency
//
// add or set a property
// typed properties have two parts : the internal value and the display value
// ie value = "display value" [+ "#" + internal_value]
//
// when setting nav aid properties, a check is done against the DB
// except if the value contain allready an internal value
//
// when a property is typed the internal value is build if there was previous
// internal value (ie the data surely come from the strachpad)
//
// properties witch name begins with a '/' are supposed to be FG properties
// and are sent to FG
//
bool DataSet(string name , string value, dataType theType)
{
	int n;
	float lat, lon, elev;
	dataEntry *x;

	// if the property is owned by FG then we set it via the prop protocol
	if( name[0] == '/') 
	{
		fmcCom->comDataSet(name, value);
		return true;
	}

    x = FindData(name);
    if( theType != -1 )
        x->type = theType;
    if(! value.empty() )
	{
        switch( x->type )
		{
		case dataTypeICAO:
            // lookup airport database
            if( (n = value.find('#')) < 0 )
			{
				if( !fmcNav->get_apt_pos(value, &lat, &lon) )
				{
					fmcPage->SetMsg(msgNIDB + value);
					return false;
				}
				// build internal value
				value += "#" + latlon(lat, lon);
			}
			// hack here
            if( name == "fmc/refapt" )
				DataSet("fmc/refapt-pos", latlon(lat, lon) , dataTypePos);
            
			break;
        case dataTypeFix:
			//TODO: there is still 100 different way to type a position in the SP

			// no internal value means the data comes from the scratchpad and must be checked
            if( (n = value.find('#')) < 0 )
			{
                n = fmcNav->get_wp_data(value, &lat, &lon, &elev);
                if( n <= 0 )
				{
					// not found
                    fmcPage->SetMsg(msgNIDB + value);
                    return false;
				}
                else if( n == 1 || !fixUnique )
				{
					// build the internal value
                    value = value + "#" + latlon(lat, lon);
				}
                else
				{
                    // non unique name, show the list
					// note that a downselect from that list will give a nav aid
					// with an internal value (so we dont iterate here)
                    fmcPage->ScratchPad = "";
                    fmcGui->NewPage(PageWpChoose);
                    return false;
                }
            }
			break;
        case dataTypePos:
            if( (n = value.find('#')) >= 0 )
                value = dataPosWp(value);
            else if( ( n=value.find(' ')) >= 0 )
			{
                value = latlon(	StringTopos(value.substr(0, n -1), false), 
								StringTopos(value.substr(n+1, value.length()-n), true));
            }
			break;
        }
    }

    x->value = value;
	return true;
}


//===========================================================================
//DataGet
//			name	= of the property to get
//			fmt		= if true then we add a nice format for the user
//
// empty formated values are set to "----"
//
// position can be converted to full user readable position
// elevation are normalized
//
string DataGet(string name, bool fmt)
{
	dataEntry *x;
	string v;

	// if the property is owned by FG then we get it via the prop protocol
	if( name[0] == '/') 
	{
		return fmcCom->comDataGet(name);
	}

    x = FindData(name);
	if( x->type < dataTypeUnk || x->type > dataTypeElev )
		x->type = dataTypeUnk;
	{
        v = x->value;
        if( fmt )
		{
			switch( x->type )
			{
            case dataTypeICAO:
                if( v.empty() )
					v = "----";
				break;
            case dataTypePos:
                v = fmtData(v, dataTypePos);
				break;
            case dataTypeElev:
                if( v.empty() )
                    v = "-----";
				break;
            default:
				if( v.empty() )
					v = "-----";
				break;
            }
        }
        return v;
    }

	return "";
}

void BatchGet(int count, char *FGproperties[], char *FMCproperties[])
{
	string result = fmcCom->BatchGet(count, FGproperties);

	vector<string> result_list = simgear::strutils::split( result, "\015\012", count ) ;
	if( result_list.size() < count )	// if no comm
		return;
	for(int n = 0 ; n < count ; n++ )
		DataSet(FMCproperties[n], result_list[n]);
}

string fmtData(string data, dataType fmt, bool showempty)
{

    if( data.empty() && !showempty )
		return "";

    if( fmt == dataTypePos )
	{
        if( data.empty() )
            return "-----'- ------'-";

		float lat, lon;
		int p = data.find('/');
		if( p >= 0 )
		{
			extract_latlon(data, &lat, &lon);
			string retval = posToString(lat, false) + " " + posToString(lon, true) + "#" + data;
			return retval;
		}

    }
	else
	{
    }

	return data;
}

string format(float n, string fmt)
{
	char temp[30];
	int txt=0;
	while( isalpha( fmt[txt] ) )
		txt ++;
	string prefix = fmt.substr(0, txt);
	string num_fmt = fmt.substr(txt);
	string out_prefix;

	if(prefix == "FL")
	{
		if( n > 10000 )
		{
			out_prefix = "FL";
			n /= 100;
			num_fmt = "---";
		}
		else
			num_fmt = "-----";
	}
	else if( prefix == "SPD" )
	{
		if( n < 1.0)
			num_fmt = ".--";
		else
			num_fmt = "---";
	}
	while(num_fmt[txt] == '-' || num_fmt[txt] == '.')
		txt ++;
	int p = num_fmt.find(".");
	if( p >= 0)
	{
		int before = p;
		int after = txt - p-1;
		sprintf(temp, "%*.*f", before, after, n);
	}
	else
	{
		sprintf(temp, "%*d", txt, (int) n);
	}
	return out_prefix + temp + num_fmt.substr(txt);
}


string format(string s, string fmt)
{
	float n;
	if( fmt == "SPD/FL" )
	{
		int p = s.find("/");
		if( p <= 0)
			return "---/-----";
		string spd = format(s.substr(0, p), "SPD");
		string fl = format(s.substr(p+1), "FL" );
		return spd + "/" + fl;
	}
	if( sscanf(s.c_str(), "%f", &n) > 0)
		return format(n, fmt);
	if( s.empty() )
		if( fmt[0] == '-')
			return fmt;
		else if( fmt == "LF" )
			return "-----";
		else
			return "---";
	return s;
}

string unformat(string s, string fmt)
{
	if(fmt.empty())
		return s;

	int p;
	if( fmt == "SPD/FL" )
	{
		string fl, spd;
		p = s.find("/");
		if( p <= 0)
		{
			fl = unformat(s, "FL" );
		}
		else
		{
			spd = unformat(s.substr(0, p), "SPD");
			fl = unformat(s.substr(p+1), "FL" );
		}
		if( spd.empty() && fl.empty() )
			return "";
		return spd + "/" + fl;
	}

	char temp[30];
	int txt=0;
	while( isalpha( fmt[txt] ) )
		txt ++;
	string prefix = fmt.substr(0, txt);
	string num_fmt = fmt.substr(txt);
	string out_prefix;
	float value, k = 1;
	int mindigit = 1, maxdigit = s.size();

	if(prefix == "FL")
	{
		if( s[0] == 'F' && s[1] == 'L' )
		{
			s = s.substr(2);
			k = 100.0;
			mindigit = maxdigit = 3;
		}
		else
		{
			mindigit = 3; maxdigit = 5;
		}
	}
	p = num_fmt.find(".");

	if( sscanf(s.c_str(), "%f", &value) < 1)
		return "";
//	if( numdigit < mindigit || numdigit > maxdigit )
//		return "";

	value *= k;

	if(p >= 0)
		sprintf(temp, "%f", value);
	else
		sprintf(temp, "%d", (int) value);
	return temp;
}

string ftos(float n)
{
	char temp[30];
	sprintf(temp, "%f", n);
	return temp;
}
float stof(string s)
{
	return atof(s.c_str());
}


//===========================================================================
//dataView
//
// extract the display value of a formated value (eliminates internal part)
//
string dataView(string data)
{
	int p = data.find('#');

	if( p >= 0)
		return data.substr( 0, p );

	return  data;
}

//===========================================================================
//dataPosWp
//
// extract the internal part or a position (eliminate user formated position)
//
string dataPosWp(string wp)
{
	int p = wp.find('#');

	if( p >= 0)
		return wp.substr( p + 1 );

	return "";
}

bool isDataEmpty(string name)
{
	string v = DataGet(name, false);
    if( v.empty() || v[0] == '-' )
        return true;

	return false;
}

string posToString(float pos , bool islon)
{
	string tmp;
	char num[16];
	int val;

    if( pos < 0 )
	{
        tmp = islon ? "W" : "S";
        pos = -pos;
    } else {
        tmp = islon ? "E" : "N";
	}

	val = (int) (pos);
	sprintf(num, islon ? "%03d" : "%02d", val);
	tmp += num;
	tmp += "";

	pos -= val; pos *= 60.0f;
	val = (int) (pos);
	sprintf(num, "%02d", val);
	tmp += num;
	tmp += "'";

	pos -= val; pos *= 60.0f;
	val = (int) (pos);
	sprintf(num, "%d", val / 10);
	tmp += num;
   
	return tmp;
}

float StringTopos(string str, bool islon)
{
	/*
Dim p As Single, neg As Boolean, n As Single
    If Left$(str, 1) = "W" Or Left$(str, 1) = "S" Then neg = True
    str = Mid$(str, 2)
    If islon Then
        p = CInt(Left$(str, 3))
        str = Mid$(str, 5)
    Else
        p = CInt(Left$(str, 2))
        str = Mid$(str, 4)
    End If
    n = CInt(Left$(str, 2)): str = Mid$(str, 4)
    p = p + n / 60
    n = CInt(Left$(str, 1))
    p = p + n / 360
    If neg Then p = -p
    StringTopos = p
	*/
	return 0.0;
}
