// fmcLegs.cxx	-- display the Leg page
//				-- modification of wp (insert, delete, direct to)
//
// Written by Harald Johnsen, started Juilly 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <simgear/math/sg_geodesy.hxx>
#include <simgear/constants.h>


#include "fmc.hxx"
#include "fmclegs.hxx"
#include "fmcpages.hxx"
#include "fmcdata.hxx"
#include "fmcnav.hxx"
#include "fmcperf.hxx"

//public
bool modLEG;

//private
static string STR="";



static SetWP(string branch, int idx, string dest, string pos, string alt, string spd)
{
	char prop[80];
	string br;

	sprintf(prop, "fmc/%s/wp[%d]", branch.c_str(), idx);
	br = prop;
    DataSet(br + "/id", dest);
    DataSet(br + "/pos", pos);
    DataSet(br + "/alt", alt);
    DataSet(br + "/spd", spd);
}



// dirty recomputation of numwp
static int WpCount()
{
	char prop[80];
	int numWP = 0;
	do
	{
        numWP ++;
		sprintf(prop, "fmc/legmod/wp[%d]/id", numWP);
    } while( ! isDataEmpty(prop) );
    return  numWP-1;
}


void FMCPage::CopyModLegsToAct(void)
{
	string dest;
	char prop[80];

    int legnr = 1;
    do
	{
		sprintf(prop, "fmc/legmod/wp[%d]", legnr);
        dest = DataGet(STR + prop + "/id", false);

        SetWP("legact", legnr, 
            dest, 
            DataGet(STR + prop + "/pos", false), 
            DataGet(STR + prop + "/alt", false), 
            DataGet(STR + prop + "/spd", false));

		if(dest.empty())
			break;

        legnr ++ ;
    } while(1);

	fmcPerf->fuel_eta_update();
	{
		float reflat, reflon;
		string pos = ( DataGet("fmc/fmc-pos", false) );
		extract_latlon ( pos, &reflat, &reflon );

		fmcNav->ehsiHelper(reflat,reflon,320,0);
	}
    modLEG = false;
}

// cancel the modifications
void FMCPage::CopyActLegsToMod(void)
{
	string dest;
	char prop[80];

    int legnr = 1;
    do
	{
		sprintf(prop, "fmc/legact/wp[%d]", legnr);
        dest = DataGet(STR + prop + "/id", false);

        SetWP("legmod", legnr, 
            dest, 
            DataGet(STR + prop + "/pos", false), 
            DataGet(STR + prop + "/alt", false), 
            DataGet(STR + prop + "/spd", false));

		if(dest.empty())
			break;

        legnr ++ ;
    } while(1);

    modLEG = false;
}

void FMCPage::LegsPage(void)
{
	string pos;
	char prop[80];
	float reflat, reflon, lat, lon;
	double az1, az2, d;

//Dim dist As Single, dir As Integer

        int numWP = WpCount();
        int np = 1 + (numWP - 1) / 5;

        int legnr = 0 + (PageNumber - 1) * 5;
        if( legnr > 0 )
		{
			sprintf(prop, "fmc/legmod/wp[%d]/pos", legnr);
            pos = DataGet( prop, false );
		}
		else
			pos = DataGet("fmc/fmc-pos", false);
//            pos = dataPosWp(previousWP);

		extract_latlon ( pos, &reflat, &reflon );
		fmcPerf->fuel_eta_update();

        if( modLEG )
            DrawPage(np, "MOD RTE LEGS");
        else
            DrawPage(np, "ACT RTE LEGS");

        for (int i = 1 ; i <= 5 ; i++ )
		{
            legnr = i + (PageNumber - 1) * 5;
            if( legnr > numWP )
				break;

			sprintf(prop, "fmc/legmod/wp[%d]", legnr);
            string dest = DataGet( STR + prop + "/id");
            pos = DataGet(STR +prop + "/pos", false);
            string alt = DataGet(STR +prop + "/alt", false);
            string spd = DataGet(STR + prop + "/spd", false);
            if( spd.empty())
				spd = "---";
			extract_latlon( pos, &lat, &lon );

            geo_inverse_wgs_84(450, reflat, reflon, lat, lon, &az1, &az2, &d);
            int dist = fmcPerf->fuel_eta_calc[legnr].dist_to_wp;
            int dir = az1;

            // now we move to next wp to calculate hdg and dist
            reflat = lat; reflon = lon;
            SetLSK (i - 1, STR + prop + "/id", ActSpecialEdit);//, dest);
			sprintf(prop, "fmc/tmp/wp[%d]", legnr);
			DataSet(prop, spd + "/" + format(alt,"FL"));
            SetLSK (i + 5, prop, ActSpecialEdit);
			char temp[80];
			sprintf(temp, "   %3dT   %4d@sNM", dir, dist);
            drawText(i * 2 - 1, DrawTextLeft, temp );
        }
		if( modLEG )
			SetLSK(L6, "<ERASE", ActSpecial, "ERALEG");

		SetLSK(R6, "*RTE DATA>", ActPage, PageRteData);
}

void FMCPage::RteDataPage(void)
{
	char prop[80];

    int numWP = WpCount();
    int np = 1 + (numWP - 1) / 5;
	int legnr = 0 + (PageNumber - 1) * 5;

	DrawPage(np, "ACT RTE DATA", " eta     wpt|fuel   wind");
	fmcPerf->fuel_eta_update();

    for (int i = 1 ; i <= 5 ; i++ )
	{
        legnr = i + (PageNumber - 1) * 5;
        if( legnr > numWP )
			break;

		sprintf(prop, "fmc/legmod/wp[%d]", legnr);
        string dest = DataGet( STR + prop + "/id");
		SetLSK (i - 1, fmcPerf->fuel_eta_calc[legnr].zulu  + " " + dest, ActViewOnly);
		SetLSK (i + 5, format(fmcPerf->fuel_eta_calc[legnr].fuel_remaining,"--.-") + "    w>", ActViewOnly);
	}

	SetLSK(L6, "<LEGS", ActPage, PageRteLegs);

}

void FMCPage::ProgressPage(void)
{
	string pos;
	char prop[80];

    int numWP = WpCount();
	int legnr;
	fmcPerf->fuel_eta_update();

	if( PageNumber == 1 )
	{
		DrawPage(2, DataGet("fmc/rtefltno") + " PROGRESS" , " to        dtg eta|fuel", 
				" next", " dest", DataGet("fmc/autopilot/active-spd-mode") + " spd|t/c", "");
		SetLSK( R6, "*POS REF>", ActPage, PageInop );

		for( int i = 1 ; i <= 3 ; i ++)
		{
			legnr = i;
			if( i == 3)
				legnr = numWP;

			sprintf(prop, "fmc/legmod/wp[%d]", legnr);
			string dest = DataGet( STR + prop + "/id");
			pos = DataGet(STR +prop + "/pos", false);
			string alt = DataGet(STR +prop + "/alt", false);
			string spd = DataGet(STR + prop + "/spd", false);

			int dist = fmcPerf->fuel_eta_calc[legnr].dist_to_go;
			float eta = fmcPerf->fuel_eta_calc[legnr].eta;

			SetLSK( i - 1, "", ActViewOnly);
			SetLSK (i - 1, STR + prop + "/id", ActDataConst, dest);
			SetLSK (i + 5, format(dist, "---- ") + fmcPerf->fuel_eta_calc[legnr].zulu + 
				format(fmcPerf->fuel_eta_calc[legnr].fuel_remaining, "--.-"), ActViewOnly);
		}
		SetLSK(L4, DataGet("fmc/autopilot/active-spd"), ActDataConst);
	}
	else
	{
		DrawPage(2, DataGet("fmc/rtefltno") + " PROGRESS" , " h/wind     wind|x/wind", 
				" xtk error|vtk error", " tas|sat", " left    fuel  used|right","","totalizer|calculated");
	}
}

//
// two cases :
//   - inserting a new wp
//   - deleting one or more wp by moving an existing wp up in the list
//
void FMCPage::PageLegsUpSelect(int LSK)
{
	char prop[80];
    string dest = ScratchPad;
    int legnr = 1 + LSK + (PageNumber - 1) * 5;
    int numWP = WpCount();
	int p = dest.find('#');
	int xleg, skip, i;

	if( ! actRte )
		return;

    if(ScratchPad != "DELETE" && (p < 0) )	// source = scrathpad
	{
        if(legnr == 1)
            fmcNav->SetNavRefpoint(previousWP);
        else
		{
            // previous leg is the refpoint
			sprintf(prop, "fmc/legmod/wp[%d]/id", legnr - 1);
            fmcNav->SetNavRefpoint( DataGet(prop) );
        }

		float lat, lon, elev;
        int n = fmcNav->get_wp_data(dest, &lat, &lon, &elev);
        if(n <= 0)
		{
            SetMsg(msgNIDB + dest);
            return;
        } else if(n >= 1)	//little hack for now, we autoseleft the first one
		{
            // ok
			string pos = latlon(lat,lon);
            dest = dest + "#" + pos;
            // now insert it

            xleg = numWP;
            for(i = numWP + 1 ; i >= legnr + 1 ; i -- )
			{
				sprintf(prop, "fmc/legmod/wp[%d]", xleg);
                SetWP("legmod", i, 
                    DataGet(STR + prop + "/id", false), 
                    DataGet(STR + prop + "/pos", false), 
                    DataGet(STR + prop + "/alt", false), 
                    DataGet(STR + prop + "/spd", false));
                    xleg -- ;
            }
            SetWP("legmod", legnr, dest, pos, "", "");
            SetWP("legmod", numWP + 2, "", "", "", "");

		} else
		{
            // NOT REACHED
            // non unique name
            ScratchPad = "";
            fmcGui->NewPage(PageWpChoose);
		}
    }
	else
	{
        if(ScratchPad == "DELETE")
            // delete this wp
            skip = 1;
        else
		{
            // deleting wp(s)
            // locate wp in the list
            for(i = legnr ; i <= numWP ; i ++ )
			{
				sprintf(prop, "fmc/legmod/wp[%d]/id", i);
                dest = DataGet(prop, false);
                if(dest == ScratchPad)
					break;
                
            }

            if( i > numWP )
			{
                SetMsg(msgINV);
                return;
            }
            skip = i - legnr;
        }

        xleg = legnr + skip;
        for(i = legnr; i <= numWP - skip ; i ++)
		{
			sprintf(prop, "fmc/legmod/wp[%d]", xleg);
            SetWP("legmod", i, 
                DataGet(STR + prop + "/id", false), 
                DataGet(STR + prop + "/pos", false), 
                DataGet(STR + prop + "/alt", false), 
                DataGet(STR + prop + "/spd", false));
			xleg ++ ;
        }
		SetWP("legmod", i, "", "", "", "");
    }

    ScratchPad = "";
    modLEG = true;
    EnableExec("MODLEG");
}


void FMCPage::ActLegs(void)
{
    CopyModLegsToAct();
}


void FMCPage::RteToLegs(void)
{
	char prop[80];
	string aRteTo, aRteVia, dest, pos;

    DataSet("fmc/rte/wp[0]/to", DataGet("fmc/rteorig", false), dataTypeUnk);
    DataSet("fmc/rte/wp[0]/via", "");
    // now we translate the route in legs
    int rtenr = 0, legnr = 0;
    do
	{
		sprintf(prop, "fmc/rte/wp[%d]", rtenr );
        aRteTo = DataGet(STR + prop + "/to", false);
        // atm via = direct because we dont handle airways or sid/star
        aRteVia = DataGet(STR + prop + "/via", false);
        rtenr ++ ;
        if(aRteTo.empty())
			break;
        if(!aRteVia.empty() )
		{
            // add all wp in this airway
        }

		pos = dataPosWp(aRteTo);
        SetWP("legmod", legnr, aRteTo, pos, "", "");
        legnr ++;
    } while(1);

    aRteTo = DataGet("fmc/rtedest");
	pos = dataPosWp(aRteTo);
    SetWP("legmod", legnr, aRteTo, pos, "", "");

    // end of list
    legnr ++;
    SetWP("legmod", legnr, "", "", "", "");

    CopyModLegsToAct();
    previousWP = DataGet("fmc/rteorig", false);

}

void FMCPage::shiftWP(void)
{
	int i;
    int numWP = WpCount();
    if(numWP > 0)
	{
        previousWP = DataGet("fmc/legact/wp[1]/id", false);
        for(i = 1 ; i < numWP ; i++)
		{
			char prop[80];
			sprintf(prop, "fmc/legact/wp[%d]", i + 1);

			SetWP("legact", i, 
				DataGet(STR + prop + "/id", false), 
				DataGet(STR + prop + "/pos", false), 
				DataGet(STR + prop + "/alt", false), 
				DataGet(STR + prop + "/spd", false));
        }
		SetWP("legact", i, "", "", "", "");
        CopyActLegsToMod();
    }
    fmcGui->Redraw();
}

