// fmcEhsi.cxx	-- a simple ehsi
//
// Written by Harald Johnsen, started Aug 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "fmc.hxx"
#include <simgear/constants.h>
#include <simgear/misc/sg_path.hxx>
#include <math.h>
#include <GL/glut.h>
#include <plib/pu.h>
#include <plib/fnt.h>
#include "fmcdata.hxx"


#define ehsiW	384
#define ehsiH	384

enum {
    ehsiModeApp,
    ehsiModeVor,
    ehsiModeMap,
    ehsiModePlan
};

enum {
	ehsiFullView,
	ehsiExpView
};

enum fntSize {
	fntSmall, fntNormal, fntBig
};

int ehsiMode = ehsiModeMap;
int ehsiView = ehsiFullView;
int ehsiRange = 10;

static string STR="";

static fntRenderer texout ;
static fntTexFont  DefaultFont;

char *rbEhsiModeLabels[] = {"APP", "VOR", "MAP", "PLN", NULL};
puButtonBox *rbEhsiMode;

char *rbEhsiRangeLabels[] = {"10", "20", "40", "80", "160", "320", NULL};
puButtonBox *rbEhsiRange;

char *rbEhsiRoseLabels[] = {"FULL", "EXP", NULL};
puButtonBox *rbEhsiRose;

static string tos(float n, int d=0)
{
	char temp[30];
	sprintf(temp, "%.*f", d, n);
	return temp;
}

static float stof(string s)
{
	return atof(s.c_str());
}

static string itos(int n, int d=1)
{
	char temp[30];
	sprintf(temp, "%0*d", d, n);
	return temp;
}

static void fontSize(fntSize size)
{
	if( size == fntSmall )
		texout . setPointSize ( 12 ) ;
	else if( size == fntBig )
		texout . setPointSize ( 18 ) ;
	else
		texout . setPointSize ( 14 ) ;
}

static void drawText(string text, float x, float y)
{
	texout.begin();
	texout.start2f ( x, y ) ;
	texout.puts( text.c_str() );
#if 0
	texout.start2f ( x+1, y ) ;
	texout.puts( text.c_str() );
#endif
	texout.end();
    glBindTexture ( GL_TEXTURE_2D, 0 ) ;

}


// TODO: replace that with a texture
void partialDiskTicked(float m_XOrigin, float m_YOrigin, float m_Radius, int center, float range, float m_DegreesPerPoint)
{
	float x, y;
	bool ten;
	float startRad, endRad, radPerPoint, currentRad, m_StartArcDegrees, m_EndArcDegrees;
	float realAngle;
	bool full = range >= 180.0;

	fontSize(fntBig);

    m_StartArcDegrees = 360.0 - range - (center % 10) + 5;
    m_StartArcDegrees = m_StartArcDegrees;
    m_EndArcDegrees = 0.0 + range + 5.0;
    if( m_EndArcDegrees - m_StartArcDegrees <= 10.0 )
        m_EndArcDegrees = m_EndArcDegrees + 360.0 ;

    startRad = m_StartArcDegrees / 180.0 * SG_PI;
    endRad = m_EndArcDegrees / 180.0 * SG_PI;
    radPerPoint = m_DegreesPerPoint / 180.0 * SG_PI;

    if (startRad >= endRad) 
		endRad = endRad + 2.0 * SG_PI;
    
    if(! full )
	{
	    glLineWidth(2.0);
		glColor3f(1.0, 1.0, 1.0);
		glBegin(GL_LINE_STRIP);
		currentRad = startRad;
		do {
			x = m_Radius * sinf(currentRad) + m_XOrigin;
			y = m_Radius * cosf(currentRad) + m_YOrigin;
			glVertex2f(x, y);
			currentRad = currentRad + radPerPoint;
		} while (currentRad < endRad);
		glEnd();
    }
    glColor3f(0.8, 0.8, 0.8);
    ten = true;
    glBegin(GL_LINES);
    currentRad = startRad;
    do
	{
        x = m_Radius * sinf(currentRad) + m_XOrigin;
        y = m_Radius * cosf(currentRad) + m_YOrigin;
		if( full )
		{
			if(ten)
			{
		        glVertex2f(x, y);
				x = (m_Radius - 10.0) * sinf(currentRad) + m_XOrigin;
				y = (m_Radius - 10.0) * cosf(currentRad) + m_YOrigin;
		        glVertex2f(x, y);
			}
		}
		else
		{
	        glVertex2f(x, y);
			if(ten)
			{
				x = (m_Radius - 20.0) * sinf(currentRad) + m_XOrigin;
				y = (m_Radius - 20.0) * cosf(currentRad) + m_YOrigin;
			}
			else
			{
				x = (m_Radius - 10.0) * sinf(currentRad) + m_XOrigin;
				y = (m_Radius - 10.0) * cosf(currentRad) + m_YOrigin;
			}
	        glVertex2f(x, y);
		}
        currentRad = currentRad + radPerPoint;
        ten = ! ten;
    } while (currentRad < endRad);
    glEnd();

	glLineWidth(1.0);

    currentRad = startRad;
    realAngle = (center / 10) * 10.0 - range + 5.0;
	float r = 45.0;
	if( full )
		r = 30.0;
    do
	{
		if( realAngle >= 360.0)
			realAngle -= 360.0;
		if( realAngle < 0.0)
			realAngle += 360.0;
        if( (((int) realAngle ) % 30) == 0.0 )
		{
			glPushMatrix();
            x = (m_Radius - r) * sinf(currentRad - 20 / 360.0) + m_XOrigin;
            y = (m_Radius - r) * cosf(currentRad - 20 / 360.0) + m_YOrigin;
			glTranslatef(x, y, 0.0);
			glRotatef(currentRad / SG_PI * 180.0,0,0,-1);
			glColor3f( 1.0, 1.0, 1.0) ;
			drawText(itos(realAngle / 10), 0, 0);
			glPopMatrix();
        }
        realAngle = (realAngle + m_DegreesPerPoint);
        currentRad = currentRad + radPerPoint;
    } while (currentRad < endRad);


}

static void draw_wpt(float x, float y)
{
    glBegin(GL_LINE_LOOP);
        glVertex2f(x - 2, y + 2);
        glVertex2f(x , y + 10);
        glVertex2f(x + 2, y + 2);
        glVertex2f(x + 10, y);
        glVertex2f(x + 2, y - 2);
        glVertex2f(x , y - 10);
        glVertex2f(x -2, y - 2);
        glVertex2f(x -10, y);
    glEnd();
}
static void draw_ndb(float x, float y)
{
    glBegin(GL_LINE_LOOP);
        glVertex2f(x - 7, y);
        glVertex2f(x - 4, y + 7);
        glVertex2f(x + 4, y + 7);
        glVertex2f(x + 7, y);
        glVertex2f(x + 4, y - 7);
        glVertex2f(x - 4, y - 7);
    glEnd();
	glBegin(GL_LINE_STRIP);
        glVertex2f(x - 7, y);
        glVertex2f(x - 10, y+2);
        glVertex2f(x - 7, y+9);
        glVertex2f(x - 4, y + 7);
    glEnd();
	glBegin(GL_LINE_STRIP);
        glVertex2f(x + 4, y + 7);
        glVertex2f(x + 7, y+9);
        glVertex2f(x + 10, y+2);
        glVertex2f(x + 7, y);
    glEnd();
	glBegin(GL_LINE_STRIP);
        glVertex2f(x + 4, y - 7);
        glVertex2f(x + 4, y - 10);
        glVertex2f(x - 4, y - 10);
        glVertex2f(x - 4, y - 7);
    glEnd();
}
static void draw_fix(float x, float y)
{
    glBegin(GL_LINE_LOOP);
        glVertex2f(x, y +20);
        glVertex2f(x - 10, y - 10);
        glVertex2f(x + 10, y - 10);
    glEnd();
}
static void draw_aid(float x, float y)
{
    glBegin(GL_LINE_LOOP);
        glVertex2f(x, y - 22);
        glVertex2f(x - 7, y - 14);
        glVertex2f(x + 7, y - 14);
    glEnd();
}
static void draw_apt(float x, float y)
{
    glBegin(GL_LINE_LOOP);
	for(int i = 0 ; i < 8 ; i ++)
	{
		float xr = sinf(i * 180 / SG_PI / 8) * 7;
		float yr = cosf(i * 180 / SG_PI / 8) * 7;
        glVertex2f(x + xr, y + yr);
	}
    glEnd();
}

static void draw_wp(float lat, float lon, float range, float diameter, float orientation)
{
	float pixel_per_NM = diameter / range;
	orientation *= SG_DEGREES_TO_RADIANS;
	float sino = sinf(orientation);
	float coso = cosf(orientation);
	int nav_count = atoi(DataGet("ehsi/nav-count").c_str());
	float lon_NM_per_deg = stof(DataGet("ehsi/lon_NM_per_deg",false));
	float lat_NM_per_deg = stof(DataGet("ehsi/lat_NM_per_deg",false));
	bool first_wp = true;
	float previous_wp_x, previous_wp_y;
	float first_wp_x, first_wp_y;
	string vor1 = DataGet("fg/radios/nav[0]/nav-id");
	string vor2 = DataGet("fg/radios/nav[1]/nav-id");
	diameter /= 2.0;

	for(int i = 0 ; i < nav_count ; i++)
	{
		char prop[80];
		sprintf(prop, "ehsi/nav[%d]", i);
		string id = DataGet(STR + prop + "/id");
		float xx = stof(DataGet(STR + prop + "/x",false));
		float yy = stof(DataGet(STR + prop + "/y",false));
		int type = atoi(DataGet(STR + prop + "/type").c_str());
		xx = (xx - lon) * lon_NM_per_deg * pixel_per_NM;
		yy = (yy - lat) * lat_NM_per_deg * pixel_per_NM;
		float x = xx*coso - yy*sino;
		float y = xx*sino + yy*coso; 
		if( type != 99 && ( fabs(xx) > diameter || fabs(yy) > diameter) )
			continue;
		switch(type)
		{
		case 2:	// NDB
			glColor3f( 0.0, 1.0, 0.0) ;
			draw_ndb(x,y);
			break;
		case 3:	// VOR
			if( id == vor1 || id == vor2 )
				glColor3f( 1.0, 1.0, 0.0) ;
			else
				glColor3f( 0.0, 1.0, 0.0) ;
			draw_ndb(x,y);
			break;
		case 40:
			glColor3f( 0.0, 0.0, 1.0) ;
			draw_apt(x,y);
			break;
		case 41:
			glColor3f( 0.0, 0.0, 1.0) ;
			draw_fix(x,y);
			break;
		case 99:
			if(first_wp)
			{
				glColor3f( 1.0, 0.0, 1.0) ;
				draw_wpt(x,y);
//				glColor3f( 1.0, 1.0, 1.0) ;
				first_wp = false;
				first_wp_x = x; first_wp_y = y;
			}
			else
			{
				glColor3f( 1.0, 0.0, 1.0) ;
				glBegin(GL_LINES);
					glVertex2f(previous_wp_x, previous_wp_y);
					glVertex2f(x, y);
				glEnd();
				glColor3f( .9, .9, .9) ;
				draw_wpt(x,y);
				glColor3f( 1.0, 1.0, 1.0) ;
			}
			previous_wp_x = x; previous_wp_y = y;
			break;
		default:
			glColor3f( 0.0, 1.0, 0.0) ;
			draw_ndb(x,y);
			break;
		}
//		drawText(id + " " + itos(type), x+10, y);
		drawText(id, x+10, y);
	}
	if(!first_wp)
	{
		glColor3f( 1.0, 0.0, 1.0) ;
		glBegin(GL_LINES);
			glVertex2f(first_wp_x, first_wp_y);
			glVertex2f(0, 0);
		glEnd();
	}
}
/*

/radios/nav/frequencies/selected-mhz
/radios/nav[1]/frequencies/selected-mhz

/radios/dme/in-range = true

/radios/nav/to-flag = 1
/radios/nav[0]/radials/selected-deg + /orientation/heading-magnetic-deg

/radios/nav/from-flag = 1
/radios/nav[0]/radials/selected-deg + /orientation/heading-magnetic-deg

(nav1)
/sim/instrument-options/hsi[0]/has-gs-needle = 1
/radios/nav[0]/gs-needle-deflection (x-shift)

(nav)
(x-shift) /radios/nav[0]/heading-needle-deflection [-10,10] * 3.2
/radios/nav[0]/radials/selected-deg - /orientation/heading-magnetic-deg

(vor1)
/radios/nav[0]/radials/actual-deg + /orientation/heading-magnetic-deg + 180
(vor2)
/radios/nav[1]/radials/actual-deg + /orientation/heading-magnetic-deg + 180
(vor)
(x-shift) /radios/nav[0]/heading-needle-deflection


*/
/*

ehsiModeVor,
	Selects heading up display, expanded compass rose, VOR navigation information, 
	elected VOR receiver, VOR frequency, VOR course, DME and TO/FROM indication. 
	Displays traffic when TFC button is enabled

ehsiModeApp
	Selects heading up, expanded compass rose, ILS navigation information, 
	selected ILS receiver, ILS frequency, ILS course and DME. 
	Displays traffic when TFC button is enabled

ehsiModeMap,
	Selects track up, expanded compass rose, and FMC generated map information, 
	airplane position and heading. 
	Also displays active waypoint data and vertical path deviation at T/D. 
	Displays traffic when TFC button is enabled

ehsiModePlan
	Selects a static, true north up, route depiction. 
	Allows route step through using the FMC-CDU page. 
	Weather radar display is inhibited. 
	TCAS traffic is not displayed


WXR - Activates and displays weather radar data. Maximum weather radar range is 320nm, although it can be displayed on 640nm range. Displayed in MAP, MAP CTR, VOR and APP ND modes. Cancels WXR FAIL messages when WXR FAIL is displayed on ND. 
STA - Displays high and low altitude navigation aids in the database if the ND range selector is in 10, 20 or 40nm range. Displays high altitude navigation aids if the ND range selector is in 80, 160, 320 or 640nm range. Displayed in MAP and MAP CTR ND modes. 
WPT - Displays waypoints in the database if the ND range selector is in 10, 20 or 40nm range. Displayed in MAP and MAP CTR ND modes. 
ARPT - Displays database airports. Displayed in MAP and MAP CTR ND modes. 
DATA - Displays ETA ETA and FMC altitude restriction at each waypoint. Displayed in MAP and MAP CTR and PLN ND modes. 
POS - Displays the IRS positions as white stars. Displays raw data VOR radials as solid green lines. These green lines extend to the DME distance to the stations tuned on the CDU NAV RADIO page. Displayed in MAP and MAP CTR ND modes

VOR/ADF Switch   
Displays VOR/ADF pointer and frequency, identifier, DME on the lower right corner of respective ND. OFF position removes the ADF and VOR displays

*/

void test1()
{
float radius;
float xCenter, yCenter, arcYcenter;

float dispHeading, trkHeading, magHeading, headingPtr;
float wpDist;
string wpName, wpEta;
float windDir, windSpeed;
float afdsDir;


	ehsiMode = rbEhsiMode->getValue();
	ehsiView = rbEhsiRose->getValue();
	ehsiRange = rbEhsiRange->getValue();
	ehsiRange = (1 << ehsiRange) * 10;

    xCenter = ehsiW / 2.0;
	windDir		= stof(DataGet("fg/environment/wind-from-heading-deg"));
	windSpeed	= stof(DataGet("fg/environment/wind-speed-kt"));
    wpDist		= stof(DataGet("ehsi/next-wp-dtg")); 
	wpName		= DataGet("ehsi/next-wp");
	wpEta		= DataGet("ehsi/next-wp-eta");
	magHeading = stof(DataGet("fg/orientation/heading-magnetic-deg"));
    trkHeading = magHeading; 
	afdsDir = stof(DataGet("fg/autopilot/settings/heading-bug-deg"));

	float tas = stof(DataGet("fg/instrumentation/airspeed-indicator/indicated-speed-kt"));	// true air speed
	float gs = stof(DataGet("fg/fdm/jsbsim/velocities/vg-fps")) * 0.592484;	// ground speed

    if(ehsiMode == ehsiModeMap)
	{
        dispHeading = trkHeading;
		headingPtr	= magHeading;
	}
    else
	{
        dispHeading = magHeading;
		headingPtr	= trkHeading;
	}

	fontSize(fntNormal);
	glColor3f( 0.8, 0.8, 0.8) ;
	drawText("GS", 4, ehsiH - 12);
	drawText("TAS", 58, ehsiH - 12);
	fontSize(fntBig);
	glColor3f( 1.0, 1.0, 1.0) ;
	drawText(itos(gs,3), 24, ehsiH - 14);
	drawText(itos(tas,3), 60+24, ehsiH - 14);
	drawText(itos(windDir,3) + "/" + itos(windSpeed,2), 4, ehsiH - 16-16);
    // wind arrow
    glPushMatrix();
		glTranslatef(35, ehsiH - 45, 0);
		glRotatef(dispHeading - windDir, 0, 0, 1);
		glColor3f(1, 1, 1);
		glBegin(GL_LINE_STRIP);
			glVertex2f(-3, 5);
			glVertex2f(0, 10);
			glVertex2f(3, 5);
		glEnd();
        glBegin(GL_LINES);
            glVertex2f(0, 10);
            glVertex2f(0, -15);
        glEnd();
    glPopMatrix();

    // ARC mode
    if(ehsiView == ehsiExpView)
	{
        radius = 240.0;
        yCenter = 0 + 50;
        arcYcenter = ehsiH - 25.0 - radius;
        yCenter = arcYcenter;
        partialDiskTicked(xCenter, arcYcenter, radius, dispHeading, 45, 5);

        // pointer
        glLineWidth(1.0);
        glBegin(GL_LINE_LOOP);
            glVertex2f(xCenter, ehsiH - 22);
            glVertex2f(xCenter - 7, ehsiH - 14);
            glVertex2f(xCenter + 7, ehsiH - 14);
        glEnd();
        // heading line
        glColor3f(0.5, 0.5, 0.5);
        glBegin(GL_LINES);
            glVertex2f(xCenter, yCenter);
            glVertex2f(xCenter, yCenter+radius);
            glVertex2f(xCenter-5, yCenter+radius / 2);
            glVertex2f(xCenter+5, yCenter+radius / 2);
        glEnd();
        glColor3f(1, 1, 1);
		fontSize(fntNormal);
		drawText(itos(ehsiRange / 2), xCenter - 24, yCenter+radius / 2 - 9);
	}
    else
	{
        // ROSE mode or plan
        radius = 146.0;
        arcYcenter = ehsiH - radius-40;
        yCenter = arcYcenter;
        partialDiskTicked(xCenter, arcYcenter, radius, dispHeading, 180, 5);
        // heading line
        glColor3f(0.5, 0.5, 0.5);
        glBegin(GL_LINES);
            glVertex2f(xCenter, yCenter+radius);
            glVertex2f(xCenter, yCenter-radius);
        glEnd();
    }

    // plane symbol
    if(ehsiView == ehsiExpView)
	    glLineWidth(2.0);
	else
	    glLineWidth(1.0);
    glColor3f(1, 1, 1);
    glBegin(GL_LINE_LOOP);
        glVertex2f(xCenter, yCenter);
        glVertex2f(xCenter - 10, yCenter - 20);
        glVertex2f(xCenter + 10, yCenter - 20);
    glEnd();
    // heading box
    glLineWidth(1.0);
	glColor3f(0.8, 0.8, 0.8);
    glBegin(GL_LINE_STRIP);
        glVertex2f(xCenter - 18, ehsiH+2);
        glVertex2f(xCenter - 18, ehsiH - 12);
        glVertex2f(xCenter + 18, ehsiH - 12);
        glVertex2f(xCenter + 18, ehsiH+2);
    glEnd();

	fontSize(fntNormal);
	glColor3f( 0.0, 1.0, 0.0) ;
	drawText(ehsiMode == ehsiModeMap ? "TRK" : "HDG", xCenter - 55, ehsiH - 12);
	drawText("MAG", xCenter + 20, ehsiH - 14);
	fontSize(fntBig);
	glColor3f( 1.0, 1.0, 1.0) ;
	drawText(itos(dispHeading,3), xCenter - 20, ehsiH - 14);

	fontSize(fntBig);
	glColor3f( 1.0, 0.0, 1.0) ;
	drawText(wpName, ehsiW - 60, ehsiH - 14);
	fontSize(fntNormal);
	drawText(wpEta + "z", ehsiW - 60, ehsiH - 26);
	drawText(tos(wpDist,1) + " NM", ehsiW - 60, ehsiH - 38);

	fontSize(fntNormal);
	glColor3f( 0.0, 1.0, 0.0) ;
	drawText("VOR 1", 4, 60);
	drawText(DataGet("fg/radios/nav[0]/nav-id"), 4, 40);
	if(DataGet("fg/radios/dme/in-range") == "true")
		drawText("DME " + tos(stof(DataGet("fg/radios/dme/distance-nm")),1), 4, 20);
	else
		drawText("---", 4, 20);

	drawText("VOR 2", ehsiW - 80 , 60);
	drawText(DataGet("fg/radios/nav[1]/nav-id"), ehsiW - 80, 40);
	drawText("---", ehsiW - 80, 20);

    // Heading Pointer
    glPushMatrix();
		glTranslatef(xCenter, arcYcenter, 0);
		glRotatef(dispHeading - headingPtr, 0, 0, 1);
		glColor3f(1, 1, 1);
		glBegin(GL_LINE_LOOP);
			glVertex2f(0.0, radius - 2);
			glVertex2f(-6, radius - 12);
			glVertex2f(6, radius - 12);
		glEnd();
    glPopMatrix();

    //AFDS Heading Bug
    glPushMatrix();
		glTranslatef(xCenter, (arcYcenter + yCenter) / 2, 0);
		glRotatef(dispHeading - afdsDir, 0, 0, 1);
		glColor3f(1, 0, 1);
		glEnable(GL_LINE_STIPPLE);
		glLineStipple(1, 0xF000);
		glBegin(GL_LINES);
			glVertex2f(0, 0);
			glVertex2f(0, radius);
		glEnd();
		glDisable(GL_LINE_STIPPLE);
		glTranslatef(0, radius, 0);
		glBegin(GL_LINE_LOOP);
			glVertex2f(-8, 0);
			glVertex2f(-8, 8);
			glVertex2f(8, 8);
			glVertex2f(8, 0);
			glVertex2f(0, 6);
		glEnd();
    glPopMatrix();

	// Nav1 pointer
	if( ehsiMode == ehsiModeVor )
	{
	    glPushMatrix();
		float navrot =  stof(DataGet("fg/radios/nav[0]/radials/selected-deg"))
					- dispHeading;

		glTranslatef(xCenter, (arcYcenter + yCenter) / 2, 0);
		glRotatef(navrot, 0, 0, 1);
		glColor3f(0, 1, 0);
		glBegin(GL_LINE_STRIP);
			glVertex2f(-10, radius);
			glVertex2f(0, radius+10);
			glVertex2f(10, radius);
		glEnd();
		glBegin(GL_LINES);
			glVertex2f(0, radius+10);
			glVertex2f(0, radius-50);
			glVertex2f(-10, radius-45);
			glVertex2f(10, radius-45);
			glVertex2f(0, -radius-10);
			glVertex2f(0, 40-radius);
		glEnd();
		glBegin(GL_LINE_STRIP);
			glVertex2f(-10, -radius-10);
			glVertex2f(0, -radius);
			glVertex2f(10, -radius-10);
		glEnd();
	    glPopMatrix();
	}

    if(ehsiMode == ehsiModeMap || ehsiMode == ehsiModePlan)
	{
		float reflat, reflon;
		string pos = DataGet("fmc/fmc-pos", false);
		extract_latlon ( pos, &reflat, &reflon );

		glPushMatrix();
			glTranslatef(xCenter, yCenter, 0);
			draw_wp(reflat, reflon, ehsiRange, radius, ehsiMode == ehsiModePlan ? 0.0 : dispHeading);
		glPopMatrix();
	}
}

static void Reshape(int x, int y)
{
	if (y == 0 || x == 0) return;
	glViewport(0,0,x,y);
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();
	glOrtho ( 0, x, 0, y, -1, 1 ) ;

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity () ;
}

static void Display(void)
{
	glClearColor ( 0.1, 0.1, 0.1, 1.0 ) ;
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();

	test1();
	puDisplay () ;

	glFlush();			//Finish rendering
	glutSwapBuffers();
}

static void motionfn ( int x, int y )
{
	puMouse ( x, y ) ;
//  glutPostRedisplay () ;
}

static void mousefn ( int button, int updown, int x, int y )
{
	puMouse ( button, updown, x, y ) ;
	glutPostRedisplay () ;
}


int InitEhsi(void)
{
	glutInitWindowSize(384+8+150,384+32);
	//Create a window with rendering context and everything else we need
	int ehsiWin = glutCreateWindow("EHSI");
	glutDisplayFunc(Display);
	glutReshapeFunc(Reshape);
	glutMouseFunc       ( mousefn   ) ;
	glutMotionFunc      ( motionfn  ) ;

	SGPath font_path( get_fg_root() );
    font_path.append( "Fonts" );

    font_path.append( "Helvetica-Narrow-Bold.txf" );

	DefaultFont.load ( font_path.c_str() );
	texout . setFont      ( & DefaultFont ) ;
	texout . setPointSize ( 20 ) ;

	// FIXME: alpha because of fonts
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER,0.5f);

	rbEhsiMode = new puButtonBox ( ehsiW + 20, 10, ehsiW + 120, 90, rbEhsiModeLabels, true ) ;
	rbEhsiRange= new puButtonBox ( ehsiW + 20, 100, ehsiW + 120, 220, rbEhsiRangeLabels, true ) ;
	rbEhsiRose = new puButtonBox ( ehsiW + 20, 230, ehsiW + 120, 270, rbEhsiRoseLabels, true ) ;
	return ehsiWin;
}
