// fmcGui.cxx -- routines for the user interface
//
// Written by Harald Johnsen, started Juilly 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include <GL/glut.h>
#include <plib/pu.h>
#include <plib/fnt.h>
#include <plib/ssg.h>
#include <simgear/misc/sg_path.hxx>
#include <simgear/misc/strutils.hxx>
#include "fmc.hxx"
#include "fmcgui.hxx"
#include "fmcpages.hxx"
#include "fmcdata.hxx"

//public
string msgEOR = "END OF ROUTE";
string msgNOFUEL = "INSUFFICIENT FUEL";
string msgNOR = "NO ACTIVE ROUTE";
string msgINV = "INVALID ENTRY ";
string msgNIDB = "NOT IN DATA BASE ";

//private

static float NormalColor[3] = {0.0f, 1.0f, 0.0f};
static float DisableColor[3] = {0.0f, 0.5f, 0.0f};
static GLuint bgTexture = 0;

//===========================================================================
//backspace_key
//
// note that the hidden content of the scratch pad is lost when it is edited
//
// if there is a message in the SP then it is cleared
//
void FMCGui::backspace_key(void)
{
	if( fmcPage->ScratchPadisMsg )
	{
		fmcPage->ScratchPad = "";
		SetMsgLed(false);
	}
	else if( fmcPage->ScratchPad == "DELETE" )
		fmcPage->ScratchPad = "";
	else if( !fmcPage->ScratchPad.empty() )
	{
		int p = fmcPage->ScratchPad.find('#');
		if( p >= 0 )
			fmcPage->ScratchPad.resize(p);
		fmcPage->ScratchPad.resize(fmcPage->ScratchPad.length()-1);
	}
	fmcPage->ScratchPadisMsg = false;
	Redraw() ;
}

static void keyboardfunc(unsigned char key, int x, int y)
{
	fmcGui->keyboardfn(key, x, y);
}

void FMCGui::keyboardfn(unsigned char key, int x, int y)
{
	if( fmcPage->ScratchPadisMsg )
	{
		return;
	}
	if( isalnum(key) || key == '.' || key == '/' || key == '-')
	{
		// TODO: delete hidden scratchpad content
		char temp[2] = { toupper(key), 0 };
		fmcPage->ScratchPad.append ( temp );
		Redraw() ;
		return;
	}
	if( key == 8 )
	{
		backspace_key();
	}
}


//===========================================================================
//genericKeycb 
//
// call the pages for the predefined keys
//
static void genericKeycallback ( puObject *thisKey )
{
	fmcGui->genericKeycb( thisKey );
}
void FMCGui::genericKeycb ( puObject *thisKey )
{
	int code = (int) thisKey->getUserData ();
//	fprintf( stderr, "button pressed code is %d\n", code );
	if( code >= L1 && code <= R6)

		fmcPage->LSKaction( code );

	else
	switch( code )
	{
		case fmcKEY_INITREF:
			NewPage(PageInitRefIndex);
			break;
		case fmcKEY_RTE:
			NewPage(PageActRte);
			break;
		case fmcKEY_DEPARR:
			NewPage(PageDepArr);
			break;
		case fmcKEY_ATC:
			break;
		case fmcKEY_VNAV:
			NewPage( PageVNAV );
			break;
		case fmcKEY_FIX:
			NewPage( PageFixInfo );
			break;
		case fmcKEY_LEGS:
			NewPage(PageRteLegs);
			break;
		case fmcKEY_HOLD:
			NewPage(PageRteHold);
			break;
		case fmcKEY_FMCCOMM:
			break;
		case fmcKEY_PROG:
			NewPage(PageProgress);
			break;
		case fmcKEY_EXEC:
			fmcPage->Execute();
			break;
		case fmcKEY_MENU:
			NewPage(PageMenu);
			break;
		case fmcKEY_NAVRAD:
			NewPage(PageNavRad);
			break;
		case fmcKEY_PREVPAGE:
			if( fmcPage->PageNumber > 1 )
			{
				fmcPage->PageNumber = fmcPage->PageNumber - 1;
				NewPage(fmcPage->currPage);
			}
			break;
		case fmcKEY_NEXTPAGE:
			if( fmcPage->PageNumber < fmcPage->PageCount )
			{
				fmcPage->PageNumber = fmcPage->PageNumber + 1;
				NewPage(fmcPage->currPage);
			}
			break;
		case fmcKEY_DEL:
			fmcPage->ScratchPad = "DELETE";
			DrawScrathPad();
			break;
		case fmcKEY_CLR:
			backspace_key();
			break;
		case fmcKEY_PLUSMINUS:
			break;
		default:
			if( isalnum(code) )
			{
				keyboardfn(code, 0, 0);
				return;
			}
			break;
	}
}


void FMCGui::addKey ( int x, int y, int w, int h, char *label, int code )
{
	buttons[code] = new puOneShot ( x, y, x+w, y+h ) ;
//	buttons[code]->setLegend ( label );
	buttons[code]->setUserData ( (void *) code );
	buttons[code]->setCallback ( genericKeycallback ) ;
}

static void Reshape(int x, int y)
{
	if (y == 0 || x == 0) return;
	glViewport(0,0,x,y);
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();
	glOrtho ( 0, x, 0, y, -1, 1 ) ;

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity () ;
}


static void motionfn ( int x, int y )
{
	puMouse ( x, y ) ;
//  glutPostRedisplay () ;
}

static void mousefn ( int button, int updown, int x, int y )
{
	puMouse ( button, updown, x, y ) ;
	glutPostRedisplay () ;
}

static void Display(void)
{
	glClearColor ( 0.1, 0.2, 0.5, 1.0 ) ;
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();

	glColor4f(1.0,1.0,1.0,1.0);
    glBindTexture(GL_TEXTURE_2D, bgTexture);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);
		glVertex2f(0,0);
		glTexCoord2f(0.0, 1.0);
		glVertex2f(0.0, 665.0);
		glTexCoord2f(1.0, 1.0);
		glVertex2f(427.0, 665.0);
		glTexCoord2f(1.0, 0.0);
		glVertex2f(427.0, 0.0);
	glEnd();
    glBindTexture(GL_TEXTURE_2D, 0);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	puDisplay () ;

	glDisable(GL_BLEND);
	fmcGui->Cls();

	fmcPage->fmcDrawPage ( fmcPage->currPage );

	glFlush();			//Finish rendering
	glutSwapBuffers();
}

static void DataTimerFunction(int n)
{
	fmcPage->DataTimerFunc(n);
	glutTimerFunc( 2000, DataTimerFunction, 0);
}
static void PerfTimerFunction(int n)
{
	fmcPage->PerfTimerFunc(n);
	glutTimerFunc( 1000, PerfTimerFunction, 2);
}

static int ehsiWin = 0;
static void ehsiTimerFunction(int n)
{
	glutPostWindowRedisplay(ehsiWin);
	glutTimerFunc( 5000, ehsiTimerFunction, 1);
}

void FMCGui::InitGui (int argc, char **argv)
{
	//Initialize GLUT
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(428,666);
	//Create a window with rendering context and everything else we need
	fmcWin = glutCreateWindow("FGFMC");
	glClearColor(0.0,0.0,0.0,0.0);
	//Assign the two used Msg-routines
	glutDisplayFunc(Display);
	glutReshapeFunc(Reshape);

	puInit () ;
    puSetDefaultStyle         ( PUSTYLE_BOXED /*PUSTYLE_SMALL_SHADED*/ );
    puSetDefaultColourScheme  (0.4, 0.3, 0.0, 0.2);


	glutMouseFunc       ( mousefn   ) ;
	glutMotionFunc      ( motionfn  ) ;
	glutKeyboardFunc	( keyboardfunc ) ;

	// because we want to change the color of the font (texture)
    glEnable (GL_COLOR_MATERIAL);
    glColorMaterial (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	// FIXME: alpha because of fonts
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER,0.1f);

	// TODO: KEYs layout, move it somewhere

#define LEFT 56
#define STEP 55
	addKey ( LEFT+STEP*0, 319, 40, 30, "INIT", fmcKEY_INITREF );
	addKey ( LEFT+STEP*1, 319, 40, 30, "RTE", fmcKEY_RTE );
	addKey ( LEFT+STEP*2, 319, 40, 30, "DEP", fmcKEY_DEPARR );
	addKey ( LEFT+STEP*3, 319, 40, 30, "ATC", fmcKEY_ATC );
	addKey ( LEFT+STEP*4, 319, 40, 30, "VNAV", fmcKEY_VNAV );

	addKey ( LEFT+STEP*0, 319-41, 40, 30, "FIX", fmcKEY_FIX );
	addKey ( LEFT+STEP*1, 319-41, 40, 30, "LEGS", fmcKEY_LEGS );
	addKey ( LEFT+STEP*2, 319-41, 40, 30, "HOLD", fmcKEY_HOLD );
	addKey ( LEFT+STEP*3, 319-41, 40, 30, "COMM", fmcKEY_FMCCOMM );
	addKey ( LEFT+STEP*4, 319-41, 40, 30, "PROG", fmcKEY_PROG );
	addKey ( LEFT+STEP*5+4, 319-41, 40, 30, "EXEC", fmcKEY_EXEC );

	addKey ( LEFT+STEP*0, 319-41*2, 40, 30, "MENU", fmcKEY_MENU );
	addKey ( LEFT+STEP*1, 319-41*2, 40, 30, "NAVRAD", fmcKEY_NAVRAD );

	addKey ( LEFT+STEP*0, 319-41*3, 40, 30, "PREV", fmcKEY_PREVPAGE );
	addKey ( LEFT+STEP*1, 319-41*3, 40, 30, "NEXT", fmcKEY_NEXTPAGE );

//	addKey ( LEFT+STEP*4, 60, 30, 25, "DEL", fmcKEY_DEL );
//	addKey ( LEFT+STEP*5, 60, 30, 25, "CLR", fmcKEY_CLR );
#undef LEFT
#undef STEP

#define LEFT 178
	for(int l = 0 ; l <= 5 ; l ++)
		for(int c = 0 ; c <= 4 ; c ++ )
		{
			int code = 65 + c + l * 5;
			if(l == 5 && c == 1)
				code = ' ';
			else if(l == 5 && c == 2)
				code = fmcKEY_DEL;
			else if(l == 5 && c == 3)
				code = '/';
			else if(l == 5 && c == 4)
				code = fmcKEY_CLR;
			addKey ( LEFT+42*c, 233 - l * 40, 30, 25, "", code );
		}
#undef LEFT

#define LEFT 9
#define STEP 33
#define TOP 568
	addKey ( LEFT, TOP-STEP*0, 24, 18, "-", L1 );
	addKey ( LEFT, TOP-STEP*1, 24, 18, "-", L2 );
	addKey ( LEFT, TOP-STEP*2, 24, 18, "-", L3 );
	addKey ( LEFT, TOP-STEP*3, 24, 18, "-", L4 );
	addKey ( LEFT, TOP-STEP*4, 24, 18, "-", L5 );
	addKey ( LEFT, TOP-STEP*5, 24, 18, "-", L6 );
#undef LEFT

#define LEFT 400
	addKey ( LEFT, TOP-STEP*0, 24, 18, "-", R1 );
	addKey ( LEFT, TOP-STEP*1, 24, 18, "-", R2 );
	addKey ( LEFT, TOP-STEP*2, 24, 18, "-", R3 );
	addKey ( LEFT, TOP-STEP*3, 24, 18, "-", R4 );
	addKey ( LEFT, TOP-STEP*4, 24, 18, "-", R5 );
	addKey ( LEFT, TOP-STEP*5, 24, 18, "-", R6 );
#undef LEFT
#undef TOP

	struct ssgTextureInfo txi;
    glEnable(GL_TEXTURE_2D);
    glGenTextures(1, &bgTexture);
    glBindTexture(GL_TEXTURE_2D, bgTexture);

	ssgLoadSGI ( "fmcbg.rgb", &txi );

    glBindTexture(GL_TEXTURE_2D, 0);

	// TODO: find some other font
	SGPath font_path( get_fg_root() );
    font_path.append( "Fonts" );
//    font_path.append( "courier_medium.txf" );
//    font_path.append( "helvetica.txf" );

    font_path.append( "Helvetica-Narrow.txf" );

	DefaultFont.load ( font_path.c_str() );
	texout . setFont      ( & DefaultFont ) ;
	texout . setPointSize ( 24 ) ;
	DefaultFont . setFixedPitch ( true );
	DefaultFont . setWidth ( 24 );
	DefaultFont . setGap ( 0 );

	ExecLedState = false;
	MsgLegState  = false;
	currentColor = NormalColor;

	extern int InitEhsi(void);
	ehsiWin = InitEhsi();
	glutTimerFunc( 2000, DataTimerFunction, 0);
	glutTimerFunc( 1000, PerfTimerFunction, 2);
	glutTimerFunc( 5000, ehsiTimerFunction, 1);
}




//===========================================================================
//FGFMCGui_Redraw
//
// ask glut for a redraw (the redraw is not done yet)
//
void FMCGui::Redraw()
{
//	glutPostRedisplay () ;
	glutPostWindowRedisplay(fmcWin);
}

//===========================================================================
//NewPage
//			num	= where to go
//
// ask the display of a new page on the CDU
// (async)
//
void FMCGui::NewPage(int num)
{
    if ( fmcPage->currPage != num )
	{
        fmcPage->PageNumber = 1;
		fmcPage->PageCount = 1;
    }
    fmcPage->currPage = num;
	Redraw() ;
}


//===========================================================================
//DrawPage
//			nbPage	= how many sub pages in this pages
//				0 => don't display a page number
//				n => display "x/n" in the top right corner of the cdu
//			title	= will be centered on the first line
//			l1, [ln]= static content of this page
//
// each line of the static content is normaly drawn in small characters
// a '|' separate the left and the right content of the line
// ie left content is left aligned, right content is right aligned
//
void FMCGui::DrawPage(int nbPage, string title, string l1, string l2, string l3, string l4, string l5, string l6)
{
	vector<string> lines = simgear::strutils::split( l1 + "\n" + l2 + "\n" + l3 + "\n" + l4 + "\n" + l5 + "\n" + l6, "\n" ) ;
	string center="            ";

	fmcPage->PageCount = nbPage;

	drawText(0, DrawTextLeft, center.substr(0, 11 - title.length() / 2) + title );

	for(int i = 0 ; i < lines.size() ; i ++)
		if(!lines[i].empty())
		{
			int p = lines[i].find("|");
			if( p >= 0 )
			{
                drawText(i * 2 + 1, DrawTextLeft, lines[i].substr(0, p ) );
                drawText(i * 2 + 1, DrawTextRight, lines[i].substr( p + 1 ) );
			}
			else
			{
				drawText(i * 2 + 1, DrawTextLeft, lines[i]);
			}
		}
	if(nbPage > 0)
	{
		char page[20];
		sprintf(page, "@s%d/%d", fmcPage->PageNumber, fmcPage->PageCount);
		drawText(0, DrawTextRight, page);
	}
}

//===========================================================================

void FMCGui::SetSmallFont()
{
    currTextSize = 0;
	texout.setPointSize(16);
}

void FMCGui::SetNormalFont()
{
    currTextSize = 1;
	texout.setPointSize(20);
}

void FMCGui::SetNormalColor()
{
	currentColor = NormalColor;
}
void FMCGui::SetDisableColor()
{
	currentColor = DisableColor;
}

#define TOP		620
#define LEFT	60


void FMCGui::Cls(void)
{
	float bgColor[4] = {0.1f, 0.1f, 0.1f, 1.0f};

	glColor4fv(bgColor);
	// must disable texture now because fnt enabled them
    glBindTexture ( GL_TEXTURE_2D, 0 ) ;

	glRectf(LEFT+7,TOP+4,LEFT+288+7,TOP-256+4);

	// draw the Exec Led
	if(ExecLedState)
		glColor3f(1.0f, 1.0f, 1.0f);
	else
		glColor3f(0.2f, 0.2f, 0.2f);

	glRectf(336+6,280, 336+30,280+10);
}

//===========================================================================
//DrawChar
//			x, y	= position on the cdu (character coordonates not pixel)
//			c		= what to display
//
// low level routine to draw a char in the current color and size
//
void FMCGui::DrawChar(int x, int y, char c)
{
	int xstep, ystep;
	float charwidth;

	c = toupper( c );
    xstep = (288 / 24);
	ystep = 17; //(256 / 14);
    x = x * xstep + 0;
    y = y * ystep + 0;
//    if(currTextSize == 0)
//        y = y + 3;

	// FIXME:this does not work well
	if ( DefaultFont.getGlyph ( c, &charwidth,
        NULL, NULL,
        NULL, NULL,
        NULL, NULL,
        NULL, NULL) )
		x += (1.0 - charwidth ) * 18.0 / 2.0;

	glColor3fv( currentColor ) ;

	texout . start2f ( x+LEFT, TOP-18 - y ) ;
	texout . putch ( c ) ;
}


//===========================================================================
//DrawScrathPad
//
// draw the content of the scratchpad on the last line
//
void FMCGui::DrawScrathPad()
{
	int ystep , y;
	string lib;

    ystep = 256 / 14;
    y = 13 * ystep + 0;
    if(fmcPage->ScratchPad[0] == '-')
		fmcPage->ScratchPad = "";

    lib = dataView(fmcPage->ScratchPad);
    drawText(13, DrawTextLeft, lib);
}


//===========================================================================
//drawText
//			ln	= line on the cdu screen
//			side = left or right
//
// odd lines default to small font sier, even to large font size
// @s and @n in the text can contredict the size of the text
//

void FMCGui::drawText(int ln, int side, char const *text)
{
	int x=0, y, i;
	char c;

    if( (ln & 1) == 0 )
        SetNormalFont();
    else
        SetSmallFont();

    y = ln;
    if( side == DrawTextRight)
	{
        x = 23 - strlen(text) + 1;
		// FIXME:should count the number of @ indeed
		if( strchr( text, '@' ) != NULL )
			x += 2;
    }
    for (i = 0 ; i < strlen(text) ; i++ )
	{
        c = toupper(text[i]);
        if( c == '@' )
		{
            i = i + 1;
	        c = text[i];
            if( c == 's' )
		        SetSmallFont();
            else if( c == 'n' )
                SetNormalFont();
            
        } else {
            DrawChar(x, y, c);
            x = x + 1;
		}
    }
}

void FMCGui::drawText(int ln, int side, string text)
{
	drawText ( ln, side, text.c_str() );
}



//===========================================================================
//
// set up the different leds
//
void FMCGui::SetMsgLed(bool state)
{

	MsgLegState = state;
	Redraw();
}

void FMCGui::SetExecLed(bool state)
{
	ExecLedState = state;
	Redraw();
}

