// fmcGui.cxx -- routines for the user interface
//
// Written by Harald Johnsen, started Juilly 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef _FMCGUI_HXX
#define _FMCGUI_HXX  1

#include <simgear/compiler.h>
#include <plib/fnt.h>
#include STL_STRING

SG_USING_STD(string);


// LSK constants
#define L1		0
#define L2		1
#define L3		2
#define L4		3
#define L5		4
#define L6		5
#define R1		6
#define R2		7
#define R3		8
#define R4		9
#define R5		10
#define R6		11
// Other special keys

typedef enum {fmcKEY_INITREF=150, fmcKEY_RTE, fmcKEY_DEPARR, fmcKEY_ATC, fmcKEY_VNAV,
fmcKEY_FIX, fmcKEY_LEGS, fmcKEY_HOLD, fmcKEY_FMCCOMM, fmcKEY_PROG, fmcKEY_EXEC,
fmcKEY_MENU, fmcKEY_NAVRAD, fmcKEY_PREVPAGE, fmcKEY_NEXTPAGE,
fmcKEY_DEL, fmcKEY_CLR, fmcKEY_PLUSMINUS, fmcKEY_BRTPLUS, fmcKEY_BRTMINUS 
} fmcKEYs;



#define	DrawTextLeft  0
#define	DrawTextRight 1


typedef enum {
    PageInop,
    PageReset,
    PageMenu,
//Pre-Flight Preparation Pages
    PageInitRefIndex,
    PageRefNavData,
    PageIdent,
    PagePosInit,
    PageActRte,
    PageDepArr,
    PageDepartures,
    PageArrivals,
    PagePerfInit,
    PageRteLegs,
//In-Flight Pages
    PageTakeoffRef,
    PageN1Lim,
    PageFixInfo,

	PageVNAV,
	PageClbCrzDes,
	PageNavRad,
//
    PageVenusWinds,
    PageRteData,
    PageOffpath,
    PageRoute, // ?
    PageWpDetails,
    PageWpChoose,
    PageFltPlans,
//Climb Pages
//    PageClbEcon,
    PageClbMaxRate,
    PageClbMaxAngle,
    PageClbEconEngOut,
    PageClbModEngOut,
//Cruise Pages
//    PageCrzEcon,
    PageCrzActEconOut,
    PageCrzModEngOut,
    PageCrzModLrc,
//Descent Pages
    PageDesPath,
    PageDesForecast,
    PagePosShift,
    PageRteHold,
    PageActPerfLimits,
    PageProgress,
    PageRtaProgress,
    PageApproachRef,
//Maintenance Pages
    PageMaintIndex,
    PageMaintComm,
    PageEnd
} PageType ;


extern string msgEOR, msgNOFUEL, msgNOR, msgINV, msgNIDB;

class puObject;
class puOneShot;
class FMCGui {

private:
	void backspace_key(void);
	void addKey ( int x, int y, int w, int h, char *label, int code );

	bool ExecLedState;
	bool MsgLegState;
	int  fmcWin;

	fntRenderer texout ;
	fntTexFont  DefaultFont;
	puOneShot *buttons[256];
	int currTextSize;
	float *currentColor;


public:

	void InitGui (int argc, char **argv);
	void Redraw();
	void NewPage(int num);
	void DrawPage(int nbPage, string title, string l1="", string l2="", string l3="", string l4="", string l5="", string l6="");
	void SetSmallFont();
	void SetNormalFont();
	void SetNormalColor();
	void SetDisableColor();
	void DrawChar(int x, int y, char c);
	void DrawScrathPad();
	void drawText(int ln, int side, char const *text);
	void drawText(int ln, int side, string text);
	void SetMsgLed(bool state);
	void SetExecLed(bool state);
	void Cls(void);



	void keyboardfn(unsigned char key, int x, int y);
	void genericKeycb ( puObject *thisKey );

};

#endif
