// fmcNav.cxx	-- access to the navigation databases
//
// Written by Harald Johnsen, started Juilly 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include "simple.hxx"
#include "runways.hxx"
#include "navdb.hxx"
#include "navlist.hxx"

#include <simgear/debug/logstream.hxx>
#include <simgear/math/sg_geodesy.hxx>
#include <simgear/misc/strutils.hxx>
#include <simgear/bucket/newbucket.hxx>

#include "fmc.hxx"
#include "fmcnav.hxx"
#include "fmcdata.hxx"
#include "fmcperf.hxx"

string fileHeader;
static string STR="";
//FIXME:this is ugly
static char fg_root[256];

char *get_fg_root()
{
	return fg_root;
}


// fmcInitNav is a copy of the code from FG (main.cxx)

/**
 * Initialize vor/ndb/ils/fix list management and query systems (as
 * well as simple airport db list)
 */
bool FMCNav::Init(string root)
{
	vector<string> nav_header;

	sglog().setLogLevels( SG_ALL, SG_INFO );
	strcpy(fg_root, root.c_str());

    SG_LOG(SG_GENERAL, SG_INFO, "Loading Simple Airport List");
    SGPath p_simple( get_fg_root() );
    p_simple.append( "Airports/basic.dat" );
    SGPath p_metar( get_fg_root() );
    p_metar.append( "Airports/metar.dat" );
    airports = new FGAirportList(p_simple.str(), p_metar.str());

    SG_LOG(SG_GENERAL, SG_INFO, "Loading Runway List");
    SGPath p_runway( get_fg_root() );
    p_runway.append( "Airports/runways.dat" );
    runways = new FGRunwayList( p_runway.str() );

    navlist = new FGNavList;
//    loclist = new FGNavList;
//    gslist = new FGNavList;
//    dmelist = new FGNavList;
//    mkrlist = new FGNavList;

//    if ( !fgNavDBInit(airports, navlist, loclist, gslist, dmelist, mkrlist) ) {
    if ( !fgNavDBInit(airports, navlist, navlist, navlist, navlist, navlist) ) {
        SG_LOG( SG_GENERAL, SG_ALERT,
                "Problems loading one or more navigational database" );
    }


    SG_LOG(SG_GENERAL, SG_INFO, "  Fixes");
    SGPath p_fix( get_fg_root() );
    p_fix.append( "Navaids/fix.dat" );
    fixlist = new FGFixList;
    fixlist->init( p_fix );

	nav_header = simgear::strutils::split( fileHeader, " ", 10 ) ;
	navcycle   = "RPEEL-" + nav_header[8].substr(0,4);
	navdate    = nav_header[6].substr(0,6);

	return true;
}

//======================================================================================


int FMCNav::search_nav( string ident)
{
    nav_list_type stations = navlist->get_navaids_ident()[ident];

    FGNavRecord *nav = NULL;
	search_result_count = stations.size();

	for ( unsigned int i = 0; i < stations.size() && i < MAX_RESULT; ++i ) {
		nav = stations[i];
		search_result[i].ident	= nav->get_ident();
		search_result[i].lat	= nav->get_lat();
		search_result[i].lon	= nav->get_lon();
		search_result[i].type	= nav->get_type();
		search_result[i].elev_ft= nav->get_elev_ft();
		search_result[i].freq	= nav->get_freq();
		search_result[i].apt_id	= nav->get_apt_id();
	}
	return search_result_count;
}




int FMCNav::search_apt( string ident)
{
	struct FGAirport apt; 
	apt = airports->search( ident );

	if( apt.id.empty() )
		return 0;
	search_result_count = 1;

	search_result[0].ident		= apt.id;
	search_result[0].lat		= apt.latitude;
	search_result[0].lon		= apt.longitude;
	search_result[0].elev_ft	= apt.elevation;
	search_result[0].freq		= 0;
	search_result[0].type		= 40;
	return 1;
}

int FMCNav::search_fix( string ident)
{
	FGFix fix;
	search_result_count = 0;
	if( !fixlist->query( ident, &fix) )
		return 0;
	search_result[0].ident		= fix.get_ident();
	search_result[0].lat		= fix.get_lat();
	search_result[0].lon		= fix.get_lon();
	search_result[0].elev_ft	= 0;
	search_result[0].freq		= 0;
	search_result[0].type		= 41;
	return 1;
}

int FMCNav::search_rwy( string ident)
{
	FGRunway rwy; // = runways->search( ident );
	search_result_count = 0;
	if(!runways->search( ident, &rwy ))
		return 0;

	do
	{
		if(rwy.id != ident)
			break;
		search_result[search_result_count].ident	= rwy.rwy_no + rwy.type;
		search_result[search_result_count].lat		= rwy.lat;
		search_result[search_result_count].lon		= rwy.lon;
		search_result[search_result_count].x		= rwy.heading;
		search_result[search_result_count].y		= rwy.length;
		search_result[search_result_count].z		= rwy.width;
		search_result[search_result_count].elev_ft	= 0;
		search_result[search_result_count].type		= 42;
		search_result_count++;
    } while (runways->next( &rwy ) );
	return search_result_count;
}

//======================================================================================


string FMCNav::nav_type_lib(int type)
{
	switch( type )
	{
		case 2: return "NDB";
		case 3: return "VOR";
		case 4: return "ILS";
		case 5: return "LOC";
		case 6: return "GS";
		case 7:
		case 8:
		case 9: return "MKR";
		case 12: return "DME";
		case 40: return "APT";
		case 41: return "FIX";
	}
	return "---";
}

bool FMCNav::get_apt_pos(string ident, float *lat, float *lon)
{
int n = search_apt(ident);

	if( ! n )
		return false;
    *lat = search_result[0].lat;
    *lon = search_result[0].lon;
    return true;
}



//===========================================================================
//get_wp_data
//			ident	= nav ident to find
//			lat,lon	= position of the nav thing
//
// search all available databases for the wanted nav
// when there is duplicates, sort the list based on distance to ref point
// search_result is filled, search_result_count will tell how many were found
//
int FMCNav::get_wp_data(string ident, float *lat, float *lon, float *elev)
{
int n;

//    ident = UCase(ident)
    n = search_fix(ident);
    if( n <= 0 )
	{
        n = search_apt(ident);
		if( n <= 0 )
		{
            n = search_nav(ident);
			if( n <= 0 )
				return 0;
            search_result_count = n;
            sortNav();
            n = search_result_count;		// count can change
        }
	}
    *lat = search_result[0].lat;
    *lon = search_result[0].lon;
	if( elev != NULL )
	    *elev = search_result[0].elev_ft;
    search_result_count = n;
    return n;
}


//===========================================================================
//SetNavRefpoint
//			value	= a position (lat/lon)
//
// set the refpoint used in the sort
//
void FMCNav::SetNavRefpoint(string value)
{
int n;
    NavRefpointPos = dataPosWp(value);
	NavRefpointlat = NavRefpointlon = 0;	// just to detect program errors
    if( NavRefpointPos.empty() )
	{
        // program logic error
		return;
	}
	n = NavRefpointPos.find('/');
	if( n < 0 )
	{
        // program logic error
		return;
	}
	sscanf("%f/%f", NavRefpointPos.c_str(), &NavRefpointlat, &NavRefpointlon );
}


//===========================================================================
//sortNav
//
// sort nav id by distance, the first one is the nearest to ref point
// ref point is previous wp in list where we want to insert this one
//
void FMCNav::sortNav(void)
{
string pos;
double az1, az2, d1, d2;

    if( NavRefpointPos.empty() )
	{
        // program logic error
        return;
	}


//   good old bubble sort

bool permut = true;
int last;
fmcRecord swap;

    last = search_result_count - 1;
    while( permut && (last > 0) )
	{
        permut = false;
        for( int i = 0 ; i < last ; i ++ )
		{
            if( search_result[i].type == 12 )
                d1 = 9.0e+30;
            else
                geo_inverse_wgs_84(450.0, NavRefpointlat, NavRefpointlon, 
					search_result[i].lat, search_result[i].lon, &az1, &az2, &d1);
            
            if( search_result[i + 1].type == 12 )
                d2 = 9.0e+30;
            else
                geo_inverse_wgs_84(450, NavRefpointlat, NavRefpointlon, 
					search_result[i + 1].lat, search_result[i + 1].lon, &az1, &az2, &d2);
            
            if( d2 < d1 )
			{
                permut = true;
                swap = search_result[i];
                search_result[i] = search_result[i + 1];
                search_result[i + 1] = swap;
            }
        }
        last -- ;
    }
	// type 12 are at the end now, we can eliminate them
    while( search_result_count > 0 && search_result[search_result_count - 1].type == 12)
        search_result_count -- ;

}

static string itos(int n)
{
	char temp[30];
	sprintf(temp, "%d", n);
	return temp;
}

struct helperData {
	float	x,y;
	string	id;
	int		type;
	float	dist;
} tbhelperData[500];

static int sortHelper(const void *a, const void *b)
{
	return (int) (((struct helperData *)a)->dist - ((struct helperData *)b)->dist);
}

static int tbhelperSize=0;

static void add_navaids(const nav_list_type *nav_tile, double lat, double lon, double range, float lat_NM_per_deg, float lon_NM_per_deg)
{
	struct helperData hd;
	FGNavRecord *nav;
	int i;
	for( i = 0 ; i < nav_tile->size() && (tbhelperSize < 500); i ++ )
	{
		nav = (*nav_tile)[i];
		hd.x = nav->get_lon();
		hd.y = nav->get_lat();
		float x = (hd.x - lon) * lon_NM_per_deg;
		float y = (hd.y - lat) * lat_NM_per_deg;
		hd.type = nav->get_type();
		if(hd.type == 2 || hd.type == 3)
		{
			hd.dist = x*x + y*y;
			if( hd.dist <= range )
			{
				hd.id = nav->get_ident();
				tbhelperData[tbhelperSize++] = hd;
			}
		}
	}
}

static void add_apt(const airport_list *nav_tile, double lat, double lon, double range, float lat_NM_per_deg, float lon_NM_per_deg)
{
	struct helperData hd;
	FGAirport *nav;
	int i;
	for( i = 0 ; i < nav_tile->size() && (tbhelperSize < 500); i ++ )
	{
		nav = (*nav_tile)[i];
		hd.x = nav->longitude;
		hd.y = nav->latitude;
		float x = (hd.x - lon) * lon_NM_per_deg;
		float y = (hd.y - lat) * lat_NM_per_deg;
		hd.type = 40;
		{
			hd.dist = x*x + y*y;
			if( hd.dist <= range )
			{
				hd.id = nav->id;
				tbhelperData[tbhelperSize++] = hd;
			}
		}
	}
}

void FMCNav::ehsiHelper(double lat, double lon, float range, int type)
{
	struct helperData hd;

	DataSet("ehsi/range", ftos(range));
	range *= range;
	tbhelperSize = 0;

	// I think the approximation is good enought for now
    SGBucket centerBucket(lon, lat);
	float	lon_NM_per_deg = centerBucket.get_width_m() / centerBucket.get_width() * SG_METER_TO_NM;
	float	lat_NM_per_deg = centerBucket.get_height_m() / centerBucket.get_height() * SG_METER_TO_NM;

    double diff;
    int lonidx = (int)lon;
    diff = lon - (double)lonidx;
    if ( (lon < 0.0) && (fabs(diff) > SG_EPSILON) ) {
		lonidx -= 1;
    }
    double lonfrac = lon - (double)lonidx;
    lonidx += 180;

    int latidx = (int)lat;
    diff = lat - (double)latidx;
    if ( (lat < 0.0) && (fabs(diff) > SG_EPSILON) ) {
		latidx -= 1;
    }
    double latfrac = lat - (double)latidx;
    latidx += 90;

    int master_index = lonidx * 1000 + latidx;

	for(int a = -1 ; a <= 1 ; a++ )
		for(int b = -1 ; b <= 1 ; b++ )
		{
			add_navaids(navlist->get_navaids_list(master_index + a + b * 1000), lat, lon, range, lat_NM_per_deg , lon_NM_per_deg  );
			add_apt(airports->get_airports_list(master_index + a + b * 1000), lat, lon, range, lat_NM_per_deg , lon_NM_per_deg  );
		}

	char prop[80];
    int legnr = 1;
    do
	{
		sprintf(prop, "fmc/legact/wp[%d]", legnr);
        string dest = dataView(DataGet(STR + prop + "/id", false));
		if(dest.empty())
			break;
		string pos = DataGet(STR + prop + "/pos", false);
		extract_latlon(pos, &hd.y, &hd.x );
		hd.dist = -500 + legnr;
		hd.id = dest;
		hd.type = 99;
		tbhelperData[tbhelperSize++] = hd;

        legnr ++ ;
    } while(legnr < 200 && (tbhelperSize < 500));
	DataSet("ehsi/next-wp", dataView(DataGet("fmc/legact/wp[1]/id", false)));
	DataSet("ehsi/next-wp-eta", fmcPerf->fuel_eta_calc[1].zulu.substr(0,4));
	DataSet("ehsi/next-wp-dtg", ftos(fmcPerf->fuel_eta_calc[1].dist_to_go));


	qsort(tbhelperData, tbhelperSize, sizeof(tbhelperData[0]), sortHelper);
	int i;
	for(i = 0 ; i < tbhelperSize && (i < 100) ; i++)
	{
		sprintf(prop, "ehsi/nav[%d]", i);
		DataSet(STR + prop + "/id", tbhelperData[i].id);
		DataSet(STR + prop + "/x",  ftos(tbhelperData[i].x));
		DataSet(STR + prop + "/y",  ftos(tbhelperData[i].y));
		DataSet(STR + prop + "/type", itos(tbhelperData[i].type));
	}
	DataSet("ehsi/nav-count", itos(i));
	DataSet("ehsi/lon_NM_per_deg", ftos(lon_NM_per_deg));
	DataSet("ehsi/lat_NM_per_deg", ftos(lat_NM_per_deg));
}


