// fmcPages.cxx	-- display the fmc pages and handle user input
//				-- store all data in the property tree and access FG when needed
//
// Written by Harald Johnsen, started Juilly 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <simgear/misc/strutils.hxx>

#include "fmc.hxx"
#include "fmcgui.hxx"
#include "fmcpages.hxx"
#include "fmcdata.hxx"
#include "fmcnav.hxx"
#include "fmclegs.hxx"
#include "fmccomm.hxx"
#include "fmcperf.hxx"

#define LINE "\n"




static string STR="";


//===========================================================================


FMCPage::FMCPage(void)
{
	currPage = PageReset;
}

//===========================================================================
//fmcDrawPage
//			num = the cdu page to display
//
// build the visual content of the pages
// also set the lsk so that the proper action will be done when they are pressed
//
// this function does not handle user input. user input is either automaticaly handled
// or the fmcSpecialEdit / fmcSpecialAction callbacks will be called
//
// standard pages have a title, some static content and some associated LSK action

void FMCPage::fmcDrawPage(int num)
{
	char prop[80];

	int i, np;
	string WpName;
	string line="------------------------";
	string perfmode = DataGet("fmc/perf/perf-mode");
	np = 1;


    for ( i = L1 ; i <= R6 ; i++ )
	{
        LSKAction[i] = ActNoOp;
		LSKFmt[i]	= "";
	}
	operatingPhase = DataGet("fmc/internal/phase", false);

	if( num == PageVNAV )
	{
		fmcGui->NewPage(PageClbCrzDes);
		if( operatingPhase == "CLB")
			PageNumber = 1;
		else if( operatingPhase == "DES")
			PageNumber = 3;
		else // if( operatingPhase == "CRZ")
			PageNumber = 2;

		return;
	}
	switch( num )
	{
    case PageReset:
		break;
    case PageMenu:
        DrawPage(0, "MENU" );
        SetLSK( 0, "*<FMC          <ACT>", ActPage, PageIdent );
		break;
//----------------------------------------------------------------------------
    case PageInitRefIndex:
        DrawPage( 1, "init/ref index");
        SetLSK( L1, "<IDENT", ActPage, PageIdent );
        SetLSK( L2, "<POS", ActPage, PagePosInit );
        SetLSK( L3, "*<PERF", ActPage, PagePerfInit );
        SetLSK( L5, "*<TAKEOFF", ActPage, PageTakeoffRef );
        SetLSK( L6, "*<APPROACH", ActPage, PageInop );//  ' PageApproachRef
        SetLSK( R1, "NAV DATA>", ActPage, PageRefNavData);
        SetLSK( R2, "*MSG RECALL>", ActPage, PageInop );
        SetLSK( R4, "*N1 LIMIT>", ActPage, PageN1Lim );
        SetLSK( R6, "*MAINT>", ActPage, PageMaintIndex );
		break;
//----------------------------------------------------------------------------
    case PageIdent:

        DrawPage(0, "IDENT" , "model|eng rating" LINE " nav data|active" LINE ""  LINE " op program"); //, line);// ', "|supp data");
        SetLSK( L1, "ac-model", ActData );
        SetLSK( L2, fmcNav->navcycle, ActViewOnly );
        SetLSK( R2, fmcNav->navdate, ActViewOnly );
        SetLSK( L4, DataGet("fmc/prog") + " (" + DataGet("fmc/prog-ver") + ")", ActViewOnly );
        SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
        SetLSK( R1, DataGet("fmc/eng-rating"), ActViewOnly );
        SetLSK( R6, "POS INIT>", ActPage, PagePosInit );
		break;
//----------------------------------------------------------------------------
    case PagePosInit:
		if( PageNumber == 1)
		{
			DrawPage(2, "POS INIT" , "|last pos" , " ref airport" , " gate" , "|set irs pos" , " utc" , line);
			SetLSK( L2, "fmc/refapt", ActData );
			SetLSK( R1, "fmc/last-pos", ActDataConst );
			SetLSK( R2, "fmc/refapt-pos", ActDataConst );
			SetLSK( R4, "fmc/irs-pos", ActData, "*" );
//			SetLSK( L5, "fmc/ref-time", ActData );
			SetLSK( L5, fmcPerf->calcTime(0,0), ActViewOnly );
			SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
			SetLSK( R6, "ROUTE>", ActPage, PageActRte );
		}
		else
		{
			DrawPage(2, "POS REF" , " fmc pos|gs" , " irs L|gs" , " irs R|gs");
			SetLSK( L1, "fmc/fmc-pos", ActData );
			SetLSK( L2, "fmc/fmc-pos", ActDataConst );
			SetLSK( L3, "fmc/fmc-pos", ActDataConst );
			SetLSK( R1, "-KT", ActViewOnly );
			SetLSK( R2, "-KT", ActViewOnly );
			SetLSK( R3, "-KT", ActViewOnly );
			SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
		}
		break;
//----------------------------------------------------------------------------
    case PagePerfInit:
//        DrawPage(0, "PERF INIT" , " gw/crz cg|trip/crz alt" LINE "fuel|crz wind" LINE 
//					"zfw|isa dev" LINE "reserves|t/c oat" LINE "cost index|trans alt");
        DrawPage(0, "PERF INIT" , " gr wt|crz alt", "fuel|cost index", 
					"zfw|min fuel temp", "reserves|crz cg", "|step size");
		{
			float grwt = stof(DataGet("fmc/perf/gr-wt"));
			float fuelwt = stof(DataGet("fmc/perf/fuel-wt"));
			float zfwt = stof(DataGet("fmc/perf/zf-wt"));
			if( fuelwt != 0.0 && zfwt != 0.0)
				DataSet("fmc/perf/gr-wt", ftos(zfwt + fuelwt));
			else if ( fuelwt != 0.0 && grwt != 0.0 && grwt > fuelwt)
				DataSet("fmc/perf/zf-wt", ftos(grwt - fuelwt));
			else if ( zfwt != 0.0 && grwt != 0.0 && grwt > zfwt)
				DataSet("fmc/perf/fuel-wt", ftos(grwt - zfwt));
		}
        SetLSK( L1, "fmc/perf/gr-wt", ActData, "*---.-");
        SetLSK( L2, "fmc/perf/fuel-wt", ActData, "*---.-" );
        SetLSK( L3, "fmc/perf/zf-wt", ActData, "*---.-" );
        SetLSK( L4, "fmc/perf/reserves-wt", ActData, "---.-" );
		SetLSK( R1, "fmc/perf/crz-alt", ActData, "*FL" );
        SetLSK( R2, "fmc/perf/costidx", ActData, "*" );
        SetLSK( R3, "fmc/perf/min-fuel-temp", ActData );
        SetLSK( R4, "fmc/perf/crz-cg", ActData, "--%");
        SetLSK( R4, "fmc/perf/step-size", ActData );
		if( DataGet("fmc/perf/fuel-wt-sensed") == DataGet("fmc/perf/fuel-wt") )
			drawText( 4, DrawTextLeft, "        LB SENSED");

//        SetLSK( R3, "fmc/perf/isadev", ActDataConst );
//        SetLSK( R4, "fmc/perf/tcoat", ActDataConst );
//        SetLSK( R5, "fmc/perf/trans-alt", ActDataConst );
        
        SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
        SetLSK( R6, "*N1 LIMIT>", ActPage, PageN1Lim );
//just a test
		fmcPerf->test_tuel();

		break;
//----------------------------------------------------------------------------
    case PageN1Lim:
        DrawPage(0, "THRUST LIM" , "|epr");
		SetLSK( L2, "*<GA", ActPage, PageInop);
		SetLSK( L3, "*<CON", ActPage, PageInop);
		SetLSK( L4, "*<CRZ", ActPage, PageInop);
        SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
		SetLSK( R2, "*CLB>", ActPage, PageInop);
		SetLSK( R3, "*CLB 1>", ActPage, PageInop);
		SetLSK( R4, "*CLB 2>", ActPage, PageInop);
        SetLSK( R6, "*TAKEOFF>", ActPage, PageTakeoffRef );

		break;
//----------------------------------------------------------------------------
    case PageTakeoffRef:
//        DrawPage(2, "takeoff ref", "flap/accell ht|v1", "e/o accell ht|vr", "thr reduction|v2", "wind/slope|trim   cg", "rwy cond|pos shift ")
        if ( ! isDataEmpty("fmc/to/flaps") )
            line = "----PRE-FLT COMPLETE-----";

        DrawPage(1, "takeoff ref" , "flaps|v1" , "thrust|vr" , "cg     trim|v2", "rwy/pos     gr wt|togw", line);
        SetLSK( L1, "fmc/to/flaps", ActData, "*--" );
        SetLSK( L2, "fmc/to/thrust-temp", ActData, "--");
        SetLSK( L3, "fmc/perf/crz-cg", ActData, "--%");
        SetLSK( L4, "fmc/rteorig-rw", ActDataConst);
		{
			float togw = stof(DataGet("fmc/to/togw"));
			float grwt = stof(DataGet("fmc/perf/gr-wt"));
			float flaps_norm = 0.125 * stof(DataGet("fmc/to/flaps")) / 5.0;
			if( togw == 0.0)
				togw = grwt;
			if( togw != 0.0 )
			{
				float vr = fmcPerf->computeVfromCl(1.5 * flaps_norm, togw * 1000.0, 0);
				DataSet("fmc/to/vr", ftos((int)vr));
			}
		}
        SetLSK( R1, "fmc/to/v1", ActSpecialEdit, "---KT" );
        SetLSK( R2, "fmc/to/vr", ActSpecialEdit, "---KT" );
        SetLSK( R3, "fmc/to/v2", ActSpecialEdit, "---KT" );
        SetLSK( R4, "fmc/to/togw", ActData, "---.-" );
		{
			char temp[80];
			sprintf(temp, "            %3.1f", stof(DataGet("fmc/perf/gr-wt")));
			drawText(8, DrawTextLeft, temp);
		}
        SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
		// normaly we show the page that is incomplete
        SetLSK( R6, "*THRUST LIM>", ActPage, PageN1Lim );
		break;
//----------------------------------------------------------------------------
    case PageRefNavData:
        DrawPage(0, "ref nav data", " ident|freq", " latitude|longitude", " mag var|elevation");
        SetLSK( L1, "fmc/tmp/navid", ActData);
		if( ! DataGet("fmc/tmp/navid", false).empty() )
		{
			string ident = dataView(DataGet("fmc/tmp/navid", false));
			float lat, lon, elev;
			int n = fmcNav->get_wp_data(ident, &lat, &lon, &elev);
			int x = 0;
			fmcRecord *s = &(fmcNav->search_result[x]);
			SetLSK( L2, posToString(lat, false), ActViewOnly );
			SetLSK( R2, posToString(lon, true), ActViewOnly );
			SetLSK( L3, "-", ActViewOnly );
			if( s->freq )
				SetLSK( R1, format(( (float)s->freq )/100.0f, "---.--"), ActViewOnly );
			if( elev > 0.0 )
				SetLSK( R3, format(elev,"-----@sFT"), ActViewOnly );
		}
        SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
		break;
//----------------------------------------------------------------------------
	case PageNavRad:
		{
			string nav1_mhz = DataGet("fmc/radios/nav1-mhz");
			string nav2_mhz = DataGet("fmc/radios/nav2-mhz");

			DrawPage(0, "NAV RADIO", " vor L|vor R ", " crs     radial|crs ", " adf L|adf R ", " ils-mls", "", "          preselect");
			SetLSK( L1, "fmc/radios/nav1-mhz",ActData,"---.--");
			SetLSK( L2, "---",  ActViewOnly );
			SetLSK( L3, DataGet("fg/instrumentation/adf/frequencies"),  ActViewOnly );
			SetLSK( L4, "---",  ActViewOnly );
			SetLSK( L6, "---",  ActViewOnly );
			SetLSK( R1, "fmc/radios/nav2-mhz",ActData,"---.--");
			SetLSK( R2, "---",  ActViewOnly );
			SetLSK( R3, "---",  ActViewOnly );
			SetLSK( R4, "---",  ActViewOnly );
			SetLSK( R6, "---",  ActViewOnly );

			char temp[80];
			sprintf(temp, "      @sm@n%s %s@sm@n", 
				DataGet("fg/radios/nav[0]/nav-id").c_str(),
				DataGet("fg/radios/nav[1]/nav-id").c_str());
			drawText(2, DrawTextLeft, temp);

			if(nav1_mhz != DataGet("fg/radios/nav[0]/frequencies/selected-mhz"))
			{
				DataSet("fg/radios/nav[0]/frequencies/selected-mhz", nav1_mhz);
				DataSet("/radios/nav[0]/frequencies/selected-mhz", nav1_mhz);
			}
			if(nav2_mhz != DataGet("fg/radios/nav[1]/frequencies/selected-mhz"))
			{
				DataSet("fg/radios/nav[1]/frequencies/selected-mhz", nav2_mhz);
				DataSet("/radios/nav[1]/frequencies/selected-mhz", nav2_mhz);
			}
		}
		break;
//----------------------------------------------------------------------------
    case PageApproachRef:
		break;
    case PageFixInfo:
        DrawPage(2, "FIX INFO" , "FIX       rad/dis FR" LINE "rad/dis    eta   dtg   alt");
        SetLSK( 0, "*HNK         071/78", ActViewOnly, "" );
        SetLSK( 1, "*050/100", ActViewOnly, "" );
		break;
    case PageProgress:
		ProgressPage();
		break;
    case PageRteHold:
        DrawPage(0, "MOD RTE HOLD" , "fix|spd/tgt alt" LINE "quad/radial|fix eta" LINE 
				"inbd crs/dir|efc time" LINE "leg time|hold avail" LINE "leg dist|best speed");
        SetLSK( 5, "*<ERASE", ActPage, PageInop );
		break;
    case PageDesForecast:
		break;
    case PageVenusWinds:
		break;
//----------------------------------------------------------------------------
    case PageActRte:
		int aWP, numWP;
        // dirty recomputation of numwp
        numWP = 0;
		do
		{
            numWP ++ ;
			sprintf(prop,"fmc/rte/wp[%d]/to", numWP);
        } while( ! isDataEmpty(prop) );
		numWP -- ;

        if( !(isDataEmpty("fmc/rteorig") || isDataEmpty("fmc/rtedest") ) )
            np = 2 + (numWP / 5);
        
        if( PageNumber > 1 )
		{
			DrawPage(np, actRte ? "ACT RTE    " : "RTE" , " via|to");
            
            for( i = 1 ; i <= 5 ; i++ )
			{
                aWP = i + (PageNumber - 2) * 5;
				sprintf(prop, "fmc/rte/wp[%d]", aWP );
                SetLSK( i - 1, STR + prop + "/via", ActDataConst , ( aWP <= numWP ? "direct" : "") );
                SetLSK( i + 5, STR + prop + "/to", ActSpecialEdit );
                if( aWP > numWP )
					break;
            }
		} else
        {
			DrawPage(np, actRte ? "ACT RTE    " : "RTE" , " origin|dest" , " co route|flt no." , " runway"); //, line);
            
            SetLSK( L1, "fmc/rteorig", ActData, "*" );
            SetLSK( R1, "fmc/rtedest", ActData, "*" );
            SetLSK( L2, "fmc/rtecoroute", ActSpecialEdit );
            SetLSK( L3, "fmc/rteorig-rw", ActData );
            SetLSK( R2, "fmc/rtefltno", ActData );
            if( actRte )
				SetLSK( L5, "*<SAVE", ActSpecial, "SAVEPLAN" );
		}
		if( actRte )
            SetLSK( R6, "*PERF INIT>", ActPage, PagePerfInit );
		else
		{
			if( !isDataEmpty("fmc/rteorig") && !isDataEmpty("fmc/rtedest") )
				SetLSK( R6, "ACTIVATE>", ActSpecial, "SETRTE" );
        }
        if( ScratchPad.empty() && isDataEmpty("fmc/rteorig") )
            ScratchPad = DataGet("fmc/refapt", false);
        
		break;
//----------------------------------------------------------------------------
    case PageRteLegs:
        LegsPage();
		break;
    case PageRteData:
		RteDataPage();
		break;
//----------------------------------------------------------------------------

    case PageDepartures:
		fmcNav->search_rwy(dataView(DataGet("fmc/rteorig")));
        np = 1 + ((fmcNav->search_result_count-1) / 5);
        for( i = 1 ; i <= 5 ; i++ )
		{
            aWP = i + (PageNumber - 1) * 5;
            if( aWP > fmcNav->search_result_count )
				break;
			fmcRecord *s = &(fmcNav->search_result[aWP - 1]);
			SetLSK( R1 + i - 1, s->ident + ">*", ActViewOnly );
		}
        DrawPage(np, dataView(DataGet("fmc/rteorig")) + " DEPARTURES", "sids|runways");
//        SetLSK( R1, "*<SEL>    " + DataGet("fmc/rteorig-rw") , ActPage, PageInop);
        SetLSK( R6, "ROUTE>", ActPage, PageActRte );
		break;
    case PageDepArr:
        DrawPage(1, "DEP/ARR INDEX");
        SetLSK( L1, "*<DEP      " + dataView(DataGet("fmc/rteorig")), ActPage, PageDepartures );
        SetLSK( L2, "*          " + dataView(DataGet("fmc/rtedest")), ActViewOnly );
        SetLSK( R1, "*ARR>", ActPage, PageInop );
        SetLSK( R2, "*ARR>", ActPage, PageArrivals );
		break;
    case PageArrivals:
        DrawPage(1, DataGet("fmc/rtedest") + " ARRIVALS" , " stars|approaches");
        SetLSK( R6, "ROUTE>", ActPage, PageActRte );
		break;
    case PageOffpath:
		break;
    case PageRoute:
		break;
//----------------------------------------------------------------------------
    case PageWpChoose:
        np = 1 + ((fmcNav->search_result_count-1) / 6);
        DrawPage(np, "SELECT DESIRED WPT");
        for( i = 1 ; i <= 6 ; i++ )
		{
            aWP = i + (PageNumber - 1) * 6;
            if( aWP > fmcNav->search_result_count )
				break;
			fmcRecord *s = &(fmcNav->search_result[aWP - 1]);
			WpName = s->ident;
			char temp[80];
			string pos = latlon(s->lat, s->lon);
			if( s->freq )
				sprintf(temp, "%3.1f ", ( (float)s->freq )/100.0f);
			else
				strcpy(temp,"         ");
			sprintf(prop,"tmp/wp[%d]", i);
			DataSet( prop, temp + fmtData(pos, dataTypePos) );
            SetLSK( i - 1, prop, ActDataConst );
            DataSet(prop, WpName + "#" + pos, dataTypeFix);
            drawText(i * 2 - 1, DrawTextLeft, (WpName + " " + fmcNav->nav_type_lib(s->type )).c_str() );
        }
		break;

//----------------------------------------------------------------------------
    case PageMaintIndex:
        DrawPage(0, "MAINT");
        SetLSK( L1, "*<FLT PLAN", ActPage, PageFltPlans );
        SetLSK( R2, "*COMM>", ActPage, PageMaintComm );
        SetLSK( L6, "<INDEX", ActPage, PageInitRefIndex );
		break;
    case PageMaintComm:
        DrawPage(0, "COMM SETUP" , "host", "port", "status", "data path");
        SetLSK( L1, fmcCom->comHost, ActViewOnly );
        SetLSK( L2, fmcCom->comPort, ActViewOnly );
        SetLSK( L3, fmcCom->comActive ? "active" : "**no comm*", ActViewOnly );
        SetLSK( L4, get_fg_root(), ActViewOnly );
        SetLSK( L6, "<MAINT", ActPage, PageMaintIndex );
		break;
//----------------------------------------------------------------------------
    case PageClbCrzDes:
		if( PageNumber == 1)
		{
			DrawPage(3, perfmode + " CLB" , "crz alt|at xxxx", perfmode + " spd|to xxxxx", " spd trans|trans alt", " spd restr|max angle", line);
			SetLSK( L1, "fmc/perf/crz-alt", ActData, "*FL" );
			SetLSK( L2, "fmc/perf/econ-spd", ActData );
			SetLSK( L3, "fmc/perf/spd-trans", ActData, "SPD/FL" );
			SetLSK( L4, "fmc/perf/spd-restr", ActData, "SPD/FL" );
			SetLSK( L5, "*<ECON", ActPage, PageInop);
			SetLSK( R1, "---/-----", ActViewOnly);
			SetLSK( R3, "fmc/perf/trans-alt", ActData, "FL" );
			if( perfmode == "EO")
				SetLSK( R5, "*ALL ENG>", ActPage, PageInop);
		}
		else if( PageNumber == 2)
		{
			DrawPage(3, perfmode + " CRZ" , "crz alt|step to", perfmode + " spd|at", "n1|eta/fuel", "step   opt   max|recmd", line);
			SetLSK( L1, "fmc/perf/crz-alt", ActData, "*FL" );
			SetLSK( L2, "fmc/perf/econ-spd", ActData );
			SetLSK( L3, "fmc/perf/tgt-n1", ActDataConst );
			SetLSK( L3, "fmc/perf/step-size", ActData );
			SetLSK( R1, "fmc/perf/step-to", ActData );
			SetLSK( R2, "", ActViewOnly );
			SetLSK( R3, "---z/   . ", ActViewOnly );
			if( perfmode != "EO")
				SetLSK( R5, "*ENG OUT>", ActPage, PageInop);
			else
				SetLSK( R5, "*ALL ENG>", ActPage, PageInop);

			SetLSK( L6, "*<RTA PROGRESS", ActPage, PageInop );
			SetLSK( R6, "*LRC>", ActPage, PageInop );
		}
		else if( PageNumber == 3)
		{
			DrawPage(3, perfmode + " DES" , "e/d at|at xxxx", perfmode + " spd", " spd trans", " spd restr", line);
			SetLSK( L1, "fmc/perf/crz-alt", ActData, "*FL" );
			SetLSK( L2, "fmc/perf/econ-spd", ActData );
			SetLSK( L3, "fmc/perf/spd-trans", ActData, "SPD/FL" );
			SetLSK( L4, "fmc/perf/spd-restr", ActData, "SPD/FL" );
			SetLSK( R1, "---/-----", ActViewOnly);
			if( operatingPhase != "DES")
				SetLSK(R6, "*DES NOW>", ActSpecial, "GODESPHASE");
		}
		break;
    case PageClbMaxRate:
        DrawPage(1, "MOD MAX RATE CLB" , " crz alt|at blaca" LINE " tgt spd|to blaca" LINE " spd rest" LINE "|clb-1 n1");
        SetLSK( 11, "*ERASE>", ActPage, PageClbCrzDes );
		break;
    case PageClbMaxAngle:
    case PageClbEconEngOut:
    case PageClbModEngOut:
		break;
    case PageCrzActEconOut:
    case PageCrzModEngOut:
    case PageCrzModLrc:
		break;
    case PageDesPath:
        DrawPage(1, perfmode + " PATH DES", "e/d alt|at ----", "tgt spd|to t/d", "spd rest|wpt alt");
        SetLSK( 5, "*<FORECAST", ActPage, PageInop );
        SetLSK( 10, "*SPD>", ActPage, PageInop );
		break;
//----------------------------------------------------------------------------
//     misc pages
    case PageFltPlans:
        DrawPage(1, "FLIGHT PLAN SUMMARY");
        DataSet("fmc/fltplan[1]", "LFSTKSFO001");
        SetLSK( L1, "fmc/fltplan[1]", ActDataConst );
        SetLSK( L2, "fmc/fltplan[2]", ActDataConst );
		break;
//----------------------------------------------------------------------------

}
    fmcGui->DrawScrathPad();


}


//===========================================================================
//DrawLSK
//			num = LSK number, L1 to L6 is drawn on the left side, R1 to R6 on the right side
//			lib = what to display
//
// if lib start with "*" then its drawn in the inactive color
// to show that the function is not available
//
void FMCPage::DrawLSK(int num , string lib )
{
int p = 0;
string moreLib;

	if( lib[0] == '*' )
	{
        fmcGui->SetDisableColor();
		lib.erase(0,1);
	}
	p = lib.find("|");
    if( p >= 0 )
	{
		moreLib = lib.substr(0, p - 1);
		lib.erase(0,p);
    }
    if( num <= L6 )
	{
		if( p >= 0 )
			drawText(num * 2 + 1, DrawTextLeft, moreLib.c_str());

        drawText(num * 2 + 2, DrawTextLeft, lib.c_str());
    } else {
		if( p >= 0 )
			drawText((num - 6) * 2 + 1, DrawTextRight, moreLib.c_str());

        drawText((num - 6) * 2 + 2, DrawTextRight, lib.c_str());
	}
    fmcGui->SetNormalColor();
}

//===========================================================================
//SetLSK
//			num = LSK (L1 to R6)
//			lib = some text or a property name if actionType == ActData*
//				the property name will be replaced by its value on the cdu
//				the property name is stored for later use (up and down select)
//			data can be some defautl value to display if the property is empty
//				or "*" meaning that the property is mendatory
//
void FMCPage::SetLSK(int num , string lib, LSKActionType actionType , string data)
{
bool oblig = false;
string fmt;

    if( data[0] == '*' )
	{
        oblig = true;
		data.erase(0,1);
	}
	fmt = data;

    LSKAction[num] = actionType;

    if( actionType == ActData || actionType == ActDataConst || actionType == ActSpecialEdit)
	{
		LSKFmt[num] = fmt;
        data = lib;
        lib = DataGet(data);
        if( oblig && lib[0] == '-' )
		{
			for(int i = 0 ; i < lib.size() ; i ++)
				if(lib[i] == '-')
					lib[i] = '=';
		}
//			lib.replace(0, "-","");	// this character is not present in the font :(

	}
    lib = dataView(lib);
	if( ! fmt.empty() )
		lib = format(lib, fmt);
    LSKData[num] = data;

	DrawLSK( num, lib );

}

//===========================================================================
//SetLSK
//			actionType	== ActPage
//			dest		= the page to go to when we tap that LSK
//
void FMCPage::SetLSK(int num , string lib, LSKActionType actionType , PageType dest)
{
	LSKPage[num] = dest;
    LSKAction[num] = actionType;
	DrawLSK( num, lib );

}



#define VERSION		"0.1"

//===========================================================================
//FMCReset
//
// initialize a few constant properties
// and initialize the type of some others
// when the properties have a type the data manager will check them
//
void FMCPage::FMCReset(void)
{
int i;
char prop[80];

    DataFixUnique(false);
    DataSet("navdata", "installation required");
    DataSet("navdata-active", "");
    DataSet("fmc/prog", "E-proto-hj");
    DataSet("fmc/prog-ver", VERSION);

    DataSet("fg/position/latitude-deg", "47");
    DataSet("fg/position/longitude-deg", "7");
    DataSet("fmc/fmc-pos", "#47/7", dataTypePos);
    DataSet("fmc/fmc-pos-alt", "");
    DataSet("fmc/last-pos", "", dataTypePos);
    DataSet("fmc/irs-pos", "", dataTypePos);

//	fmcPerf_CheckPos();

    DataSet("fmc/refapt", "", dataTypeICAO);
    DataSet("fmc/refapt-pos", "", dataTypePos);

    DataSet("fmc/rteorig", "", dataTypeICAO);
    DataSet("fmc/rteorig-rw", "");
    DataSet("fmc/rtedest", "", dataTypeICAO);

	DataSet("fmc/tmp/navid", "", dataTypeFix);

	DataSet( "fmc/perf/gr-wt", "");
	DataSet( "fmc/perf/fuel-wt", "");
	DataSet( "fmc/perf/zf-wt", "");
	DataSet( "fmc/perf/reserves-wt", "");

	float sensed_fuel = 0;
	for(i = 0; i <= 5 ; i ++)
	{
		sprintf(prop, "/consumables/fuel/tank[%d]/level-lb", i);

		string fuel_in_tank = DataGet(prop);
		sensed_fuel += atof( fuel_in_tank.c_str() );
	}
	sensed_fuel = sensed_fuel / 1000.0f;

	DataSet( "fmc/perf/fuel-wt-sensed", ftos(sensed_fuel));
	DataSet( "fmc/perf/fuel-wt", ftos(sensed_fuel));

    DataSet("ac-model", DataGet("/sim/aircraft"));

//    DataGet ("/sim/flight-model") ' jsb|yasim
//    DataGet ("/sim/aero")          ' 737
// /sim/sceneryloaded (True)
// /environment/temperature-degc (15.000000)
// /environment/magnetic-variation-deg (0.406728)
// /consumables/fuel/tank/level-lb (9917.425467)


    for(i = 0 ; i < 30 ; i++ )
	{
		sprintf(prop, "fmc/rte/wp[%d]/to", i);
        DataSet(prop, "", dataTypeFix);
		sprintf(prop, "fmc/rte/wp[%d]/via", i);
        DataSet(prop, "", dataTypeFix);
    }
    for(i = 0 ; i < 50 ; i++ )	// route converted to legs
	{
		sprintf(prop, "fmc/legmod/wp[%d]/id", i);
        DataSet(prop, "", dataTypeUnk);
    }
    for(i = 0 ; i < 3 ; i ++ )	// active route (legs)
	{
		sprintf(prop, "fmc/legact/wp[%d]/id", i);
        DataSet(prop, "", dataTypeUnk);
    }

	DataSet( "fmc/perf/trans-alt", "18000" );
	DataSet( "fmc/perf/perf-mode", "econ" );
	DataSet("fmc/internal/phase", "CLB");

	operatingPhase = "";
    ScratchPadisMsg = false; ScratchPad = "";
    currPage = -1;
    PageNumber = 1;
    actRte = false;
    DataFixUnique(true);
    fmcGui->NewPage(PageIdent);
}



//===========================================================================
//EnableExec
//			action	= what will be done when exec is pressed
//
void FMCPage::EnableExec(string action)
{
    ExecFunction = action;
    fmcGui->SetExecLed(true);
}


//===========================================================================
//fmcSpecialEdit
//			LSK = LSK (L1 to R6)
//
// some data can't just be up selected so simply because of type checking
// or because we want to do something special
// we come here when the LSK was created with ActSpecialEdit actionType
//
void FMCPage::SpecialEdit(int LSK)
{
string value = ScratchPad;
char prop[80];

    switch(currPage)
	{
    case PageRteLegs: 
		if( LSK >= L1 && LSK <= L6 )
			PageLegsUpSelect(LSK);
		else
		{
			int legnr = LSK - 5 + (PageNumber - 1) * 5;
			sprintf(prop, "fmc/legmod/wp[%d]", legnr);
			string value = unformat(ScratchPad, "SPD/FL");
			if( ! value.empty() )
			{
				int p = value.find("/");
				if( p <= 0)
					return;
				string spd = value.substr(0,p);
				string alt = value.substr(p+1);
				if(!alt.empty())
					DataSet(STR + prop + "/alt", alt);
				if(!spd.empty())
					DataSet(STR + prop + "/spd", spd);
				ScratchPad = "";
			}
			else
				SetMsg(msgINV);
		}
		break;

    case PageActRte:
        if( PageNumber == 1 )
		{
			// this is the coroute LSK
			// load the flight plan from disk ?
			// and build the list of WP on the RTE page
            if( LSK == L2 )
			{
//				string aRouteText = "KDFW.TRISS.TRO.SHERO.MATIE.SWAPP.KLGA";
//				string aRouteText = "LFST.MIRGU.TIRSO.ARPUS.EPOXI.TORPA.MOROK.GILIR.TUROM.BOLGI.MILPA.KOGAS.VANAS.MEDAM.TOSMI.VEVAR.LFMN";
//				string aRouteText = "LFST.NATOR.TITIX.TRA.RIPUS.GERSA.MALUS.DEGAD.ODINA.EVANO.ENOBA.BORDI.PIGOS.LFMN";
				// LFMN DCT TOP A1 SPR W103 LPS W102 LASUN DCT LFSB
				string aRouteText = "LFMN.TOP.KERES.ADISO.BANKO.SPR.LPS.BALIR.LEPLA.LASUN.LFSB";
				vector<string> aRouteList = simgear::strutils::split( aRouteText, "." ) ;

                DataSet("fmc/rtecoroute", value);
                ScratchPad = "";
				// little hack, I don't want to handle duplicates when loading a plan
                DataFixUnique(false);
                DataSet( "fmc/refapt", aRouteList[0] );
                DataSet( "fmc/rteorig", aRouteList[0] );
                fmcNav->SetNavRefpoint( DataGet("fmc/rteorig") );
                int legnr = 1;
                for( int i = 1 ; i < aRouteList.size()-1 ; i ++ )
				{
                    if( !aRouteList[i].empty() )
					{
                        if( legnr > 1 )
						{
							sprintf(prop, "fmc/rte/wp[%d]/to", legnr - 1 );
                            fmcNav->SetNavRefpoint( DataGet( prop ) );
						}
						sprintf(prop, "fmc/rte/wp[%d]/to", legnr );
                        DataSet( prop, aRouteList[i]);
                        legnr ++ ;
                    }
                }
				actRte = false;
                DataSet("fmc/rtedest", aRouteList[i] );
				sprintf(prop, "fmc/rte/wp[%d]/to", legnr );
                DataSet( prop, "");
				DataSet("fmc/fmc-pos", DataGet("fmc/rteorig"));
                DataFixUnique(true);
                SetMsg(value + " LOADED");
            }
        }
		else
		{
            // addition of a nav point
            // set reference point first for the good sorting of things
			// because if the wp name is ambiguous we will display the "choose wp page"
            int legnr = 1 + LSK - R1 + (PageNumber - 2) * 5;
            if( legnr == 1 )
                // rte orig is the refpoint
                fmcNav->SetNavRefpoint(DataGet("fmc/rteorig"));
            else
			{
                // previous segment is the refpoint
				sprintf(prop, "fmc/rte/wp[%d]/to", legnr - 1);
                fmcNav->SetNavRefpoint(DataGet(prop));
            }
            // lets add it
			sprintf(prop, "fmc/rte/wp[%d]/to", legnr);
			ScratchPad = "";
        }
		break;

    case PageTakeoffRef:
/*        If LSK = R1 Then        ' compute v1
        ElseIf LSK = R2 Then    ' compute vr
        ElseIf LSK = R3 Then    ' compute v2
        End If*/
		break;
    }
    fmcGui->Redraw();
}


//===========================================================================
//fmcSpecialAction
//			action	= what to do
//
// 
void FMCPage::SpecialAction(string action)
{
	if( action == "SETRTE" )
	{
		EnableExec(action);
	}
	else if( action == "SAVEPLAN" )
	{
		// noop
		string filename = dataView(DataGet("fmc/rteorig")) + 
							dataView(DataGet("fmc/rtedest")) +
							"001";
		SetMsg(filename + " SAVED");
	}
	else if( action == "LOADPLAN" )
	{
		// noop
	}
	else if( action == "ERALEG" )
	{
		CopyActLegsToMod();
	}
	else if( action == "GODESPHASE" )
	{
		DataSet("fmc/internal/phase", "DES");
		// set ap des to 1250 ft/min until planned des path
	}
}


//===========================================================================
//fmcExecute
//
// called when the user tap the EXEC key
//
void FMCPage::Execute()
{
    if( ExecFunction == "SETRTE" )
	{
            RteToLegs();
            actRte = true;
	}
	else if( ExecFunction == "MODLEG" )
	{
            ActLegs();
    }
    fmcGui->SetExecLed(false);
    ExecFunction = "";
}



//===========================================================================
//FGFMCPage_LSKaction
//			index = LSK L1 to R6
//
// called when the user tap one of the LSK
//
// depending on the type of LSK
//	- ActPage		=> go to a new page
//	- ActData		=> can do up or down select
//	- ActDataConst	=> can only do down select (data is view only)
//	- ActSpecialEdit=> can do normal down select but up select is a special function
//	- ActSpecial	=> no up or down select
//
// up or down select if determined by the content of the scrathpad
//
void FMCPage::LSKaction(int index)
{

	switch ( LSKAction[index] )
	{
	case ActPage:

        if( LSKPage[index] != PageInop )
            fmcGui->NewPage( LSKPage[index] );

		break;

    case ActData:
        if( !ScratchPadisMsg )
		{
            if( ScratchPad.empty() )	// down select
			{
                ScratchPad = DataGet( LSKData[index] );
			}
            else   // up select
			{
                if( ScratchPad == "DELETE" ) 
				{
					ScratchPad = "";
					DataSet(LSKData[index], ScratchPad);
					break;
				}
				string value = unformat(ScratchPad, LSKFmt[index]);
				if( value.empty() )
					SetMsg(msgINV);
				else
					if( DataSet(LSKData[index], value) )
					{
						ScratchPad = "";
					}
            }
        }
		break;

    case ActSpecialEdit:

        if( !ScratchPadisMsg )
		{
            if( ScratchPad.empty() )		// down select
			{
                ScratchPad = DataGet(LSKData[index]);
			}
			else
			{		//up select
                SpecialEdit(index);
            }
        }
		break;

    case ActDataConst:	// down select only
        if( !ScratchPadisMsg )
		{
			if( ScratchPad.empty() )	// down select
			{
				ScratchPad = DataGet(LSKData[index]);
			}
        }
		break;

    case ActSpecial:
        SpecialAction(LSKData[index]);
		break;

	}
	fmcGui->Redraw();
}


//===========================================================================
//SetMsg
//			msg	= what to display in the scrathpad
//
// display a new message in the scratchpad if it is empty,
// if not then add it to the pending message list
// 
void FMCPage::SetMsg(string msg)
{

    // put message in the scratchpad
    if( ScratchPad.empty() )
	{
        ScratchPad = msg;
        ScratchPadisMsg = true;
	}
    else	//add message in message pad
	{
		char prop[80];
        for( int i = 1 ; i <= 5 ; i++ )
		{
			sprintf(prop, "fmc/msg[%d]", i);
            if( DataGet(prop).empty() )
			{
                DataSet(prop, msg);
                fmcGui->SetMsgLed(true);
                break;
            }
        }
    }

	fmcGui->Redraw();
}


void FMCPage::DataTimerFunc(int n)
{
	static bool comset=true;
	char *FGprop[]={"/position/latitude-deg","/position/longitude-deg", "/position/altitude-agl-ft",
		"/environment/wind-from-heading-deg","/environment/wind-speed-kt",
		"/orientation/heading-magnetic-deg", "/instrumentation/airspeed-indicator/indicated-speed-kt",
		"/autopilot/gui/hdg-lnav", "/autopilot/gui/alt-vnav", "/autopilot/gui/vel-vnav",
		"/autopilot/gui/hdg-active", "/autopilot/gui/alt-active", "/autopilot/gui/vel-active",
		"/radios/nav[0]/frequencies/selected-mhz","/radios/nav[0]/nav-id","/radios/nav[0]/to-flag"
	};
	char *FMCprop[]={"fg/position/latitude-deg","fg/position/longitude-deg", "fg/position/altitude-ft",
		"fg/environment/wind-from-heading-deg","fg/environment/wind-speed-kt",
		"fg/orientation/heading-magnetic-deg", "fg/instrumentation/airspeed-indicator/indicated-speed-kt",
		"fg/autopilot/gui/hdg-lnav", "fg/autopilot/gui/alt-vnav", "fg/autopilot/gui/vel-vnav",
		"fg/autopilot/gui/hdg-active", "fg/autopilot/gui/alt-active", "fg/autopilot/gui/vel-active",
		"fg/radios/nav[0]/frequencies/selected-mhz","fg/radios/nav[0]/nav-id","fg/radios/nav[0]/to-flag"
	};
	char *FGprop2[]={
		"/radios/nav[0]/radials/selected-deg","/radios/nav[0]/from-flag","/radios/nav[0]/gs-needle-deflection","/radios/nav[0]/heading-needle-deflection",
		"/radios/nav[1]/frequencies/selected-mhz","/radios/nav[1]/nav-id","/radios/nav[1]/to-flag",
		"/radios/nav[1]/radials/selected-deg","/radios/nav[1]/from-flag","/radios/nav[1]/gs-needle-deflection","/radios/nav[1]/heading-needle-deflection",
		"/radios/dme/in-range","/radios/dme/distance-nm","/fdm/jsbsim/velocities/vg-fps",
		"/instrumentation/adf/frequencies"
	};
	char *FMCprop2[]={
		"fg/radios/nav[0]/radials/selected-deg","fg/radios/nav[0]/from-flag","fg/radios/nav[0]/gs-needle-deflection","fg/radios/nav[0]/heading-needle-deflection",
		"fg/radios/nav[1]/frequencies/selected-mhz","fg/radios/nav[1]/nav-id","fg/radios/nav[1]/to-flag",
		"fg/radios/nav[1]/radials/selected-deg","fg/radios/nav[1]/from-flag","fg/radios/nav[1]/gs-needle-deflection","fg/radios/nav[1]/heading-needle-deflection",
		"fg/radios/dme/in-range","fg/radios/dme/distance-nm","fg/fdm/jsbsim/velocities/vg-fps",
		"fg/instrumentation/adf/frequencies"
	};
//	if(!fmcCom->comActive)
//		return;
	if( sizeof(FGprop) != sizeof(FMCprop) || sizeof(FGprop2) != sizeof(FMCprop2))
	{
		return;
	}
	if(comset)
		BatchGet(sizeof(FGprop) / sizeof(char *), FGprop, FMCprop);
	else
		BatchGet(sizeof(FGprop2) / sizeof(char *), FGprop2, FMCprop2);
	comset = !comset;

	string lat, lon;
	lat = DataGet("fg/position/latitude-deg");
	lon = DataGet("fg/position/longitude-deg");
	DataSet("fmc/fmc-pos", "#" + lat + "/" + lon);

	DataSet("fmc/radios/nav1-mhz", DataGet("fg/radios/nav[0]/frequencies/selected-mhz"));
	DataSet("fmc/radios/nav2-mhz", DataGet("fg/radios/nav[1]/frequencies/selected-mhz"));
	fmcPerf->fuel_eta_update();
	fmcNav->ehsiHelper(stof(lat),stof(lon),320,0);
	fmcGui->Redraw();
}

void FMCPage::PerfTimerFunc(int n)
{
	fmcPerf->update();
	extern void AP_update(void);

	AP_update();

}
