// fmcperf.cxx	-- compute fuel, distance, eta
//
// Written by Harald Johnsen, started Aug 2004.
//
// Copyright (C) 2004  Harald Johnsen  - hjohnsen@evc.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


struct fuel_eta_calc_struct {
	float dist_to_go;
	float dist_to_wp;
	float eta;
	string zulu;
	float fuel_remaining;
	float speed;
	float alt;
	float weight;
};



class FMCPerf {

private:

	float fdmS;			//    wing area
	float fdmCLalpha;	//     'per radian
	float fdmCL0;
	float fdmCLmax;
	float fdmCD0;
	float fdmK;
	float fdmTSFC;

public:
	FMCPerf();
	void init(void);
	float calcCL(float Lift , float rho , float v , float S);
	float CalcCd(float cdo, float K, float cl, float M);
	float alt_dens(float h);
	float alt_temp(float h);	// in Rankin
	float velocity_mach(float v, float h);
	float calcMach(float spd, float temp);
	float compute_fuel(float v, float alt, float w);
	float Cost_Function(float vk, float altft, float wlb);
	float query_wp_speed(string wp);
	float query_wp_alt(string wp);

	float time_needed(float d, float v);
	float fuel_needed(float ff, float t);
	void test_tuel(void);

	void update(void);
	void fuel_eta_update(void);
	float computeVfromCl(float cl, float w, float h);
	string calcTime( int ct, long int warp ) ;

	struct fuel_eta_calc_struct fuel_eta_calc[200];

};
